@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2017 Mort Bay Consulting Pty. Ltd.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -19,6 +19,8 @@
 package org.eclipse.jetty.client;
 
 import java.nio.ByteBuffer;
+import java.util.concurrent.Executor;
+import java.util.concurrent.RejectedExecutionException;
 import java.util.concurrent.atomic.AtomicReference;
 
 import org.eclipse.jetty.client.api.ContentProvider;
@@ -37,16 +39,16 @@ import org.eclipse.jetty.util.log.Logger;
  * the transport-specific code to send requests over the wire, implementing
  * {@link #sendHeaders(HttpExchange, HttpContent, Callback)} and
  * {@link #sendContent(HttpExchange, HttpContent, Callback)}.
- * <p />
+ * <p>
  * {@link HttpSender} governs two state machines.
- * <p />
+ * <p>
  * The request state machine is updated by {@link HttpSender} as the various steps of sending a request
- * are executed, see {@link RequestState}.
+ * are executed, see {@code RequestState}.
  * At any point in time, a user thread may abort the request, which may (if the request has not been
- * completely sent yet) move the request state machine to {@link RequestState#FAILURE}.
+ * completely sent yet) move the request state machine to {@code RequestState#FAILURE}.
  * The request state machine guarantees that the request steps are executed (by I/O threads) only if
  * the request has not been failed already.
- * <p />
+ * <p>
  * The sender state machine is updated by {@link HttpSender} from three sources: deferred content notifications
  * (via {@link #onContent()}), 100-continue notifications (via {@link #proceed(HttpExchange, Throwable)})
  * and normal request send (via {@link #sendContent(HttpExchange, HttpContent, Callback)}).
@@ -63,7 +65,7 @@ public abstract class HttpSender implements AsyncContentProvider.Listener
     private final AtomicReference<SenderState> senderState = new AtomicReference<>(SenderState.IDLE);
     private final Callback commitCallback = new CommitCallback();
     private final IteratingCallback contentCallback = new ContentCallback();
-    private final Callback lastCallback = new LastContentCallback();
+    private final Callback lastCallback = new LastCallback();
     private final HttpChannel channel;
     private HttpContent content;
     private Throwable failure;
@@ -83,6 +85,11 @@ public abstract class HttpSender implements AsyncContentProvider.Listener
         return channel.getHttpExchange();
     }
 
+    public boolean isFailed()
+    {
+        return requestState.get() == RequestState.FAILURE;
+    }
+
     @Override
     public void onContent()
     {
@@ -173,7 +180,8 @@ public abstract class HttpSender implements AsyncContentProvider.Listener
         if (expects100Continue(request))
             newSenderState = content.hasContent() ? SenderState.EXPECTING_WITH_CONTENT : SenderState.EXPECTING;
 
-        out: while (true)
+        out:
+        while (true)
         {
             SenderState current = senderState.get();
             switch (current)
@@ -330,18 +338,34 @@ public abstract class HttpSender implements AsyncContentProvider.Listener
         }
     }
 
-    protected boolean anyToFailure(Throwable failure)
+    private void anyToFailure(Throwable failure)
     {
         HttpExchange exchange = getHttpExchange();
         if (exchange == null)
-            return false;
+            return;
+
+        if (LOG.isDebugEnabled())
+            LOG.debug("Request failure " + exchange.getRequest(), failure);
 
         // Mark atomically the request as completed, with respect
         // to concurrency between request success and request failure.
         if (exchange.requestComplete(failure))
-            return abort(exchange, failure);
+            executeAbort(exchange, failure);
+    }
 
-        return false;
+    private void executeAbort(HttpExchange exchange, Throwable failure)
+    {
+        try
+        {
+            Executor executor = getHttpChannel().getHttpDestination().getHttpClient().getExecutor();
+            executor.execute(() -> abort(exchange, failure));
+        }
+        catch (RejectedExecutionException x)
+        {
+            if (LOG.isDebugEnabled())
+                LOG.debug(x);
+            abort(exchange, failure);
+        }
     }
 
     private void terminateRequest(HttpExchange exchange)
@@ -376,6 +400,7 @@ public abstract class HttpSender implements AsyncContentProvider.Listener
         }
         else
         {
+            result = channel.exchangeTerminating(exchange, result);
             HttpDestination destination = getHttpChannel().getHttpDestination();
             boolean ordered = destination.getHttpClient().isStrictEventOrdering();
             if (!ordered)
@@ -392,7 +417,7 @@ public abstract class HttpSender implements AsyncContentProvider.Listener
     /**
      * Implementations should send the HTTP headers over the wire, possibly with some content,
      * in a single write, and notify the given {@code callback} of the result of this operation.
-     * <p />
+     * <p>
      * If there is more content to send, then {@link #sendContent(HttpExchange, HttpContent, Callback)}
      * will be invoked.
      *
@@ -404,11 +429,11 @@ public abstract class HttpSender implements AsyncContentProvider.Listener
 
     /**
      * Implementations should send the content at the {@link HttpContent} cursor position over the wire.
-     * <p />
-     * The {@link HttpContent} cursor is advanced by {@link HttpSender} at the right time, and if more
+     * <p>
+     * The {@link HttpContent} cursor is advanced by HttpSender at the right time, and if more
      * content needs to be sent, this method is invoked again; subclasses need only to send the content
      * at the {@link HttpContent} cursor position.
-     * <p />
+     * <p>
      * This method is invoked one last time when {@link HttpContent#isConsumed()} is true and therefore
      * there is no actual content to send.
      * This is done to allow subclasses to write "terminal" bytes (such as the terminal chunk when the
@@ -509,7 +534,8 @@ public abstract class HttpSender implements AsyncContentProvider.Listener
     {
         // Update the state to avoid more request processing.
         boolean terminate;
-        out: while (true)
+        out:
+        while (true)
         {
             RequestState current = requestState.get();
             switch (current)
@@ -536,7 +562,7 @@ public abstract class HttpSender implements AsyncContentProvider.Listener
 
         Request request = exchange.getRequest();
         if (LOG.isDebugEnabled())
-            LOG.debug("Request failure {} {} on {}: {}", request, exchange, getHttpChannel(), failure);
+            LOG.debug("Request abort {} {} on {}: {}", request, exchange, getHttpChannel(), failure);
         HttpDestination destination = getHttpChannel().getHttpDestination();
         destination.getRequestNotifier().notifyFailure(request, failure);
 
@@ -546,14 +572,14 @@ public abstract class HttpSender implements AsyncContentProvider.Listener
             // respect to concurrency between request and response.
             Result result = exchange.terminateRequest();
             terminateRequest(exchange, failure, result);
+            return true;
         }
         else
         {
             if (LOG.isDebugEnabled())
                 LOG.debug("Concurrent failure: request termination skipped, performed by helpers");
+            return false;
         }
-
-        return true;
     }
 
     private boolean updateRequestState(RequestState from, RequestState to)
@@ -581,11 +607,11 @@ public abstract class HttpSender implements AsyncContentProvider.Listener
     public String toString()
     {
         return String.format("%s@%x(req=%s,snd=%s,failure=%s)",
-                getClass().getSimpleName(),
-                hashCode(),
-                requestState,
-                senderState,
-                failure);
+            getClass().getSimpleName(),
+            hashCode(),
+            requestState,
+            senderState,
+            failure);
     }
 
     /**
@@ -708,7 +734,7 @@ public abstract class HttpSender implements AsyncContentProvider.Listener
             if (!headersToCommit(exchange))
                 return;
 
-             HttpContent content = HttpSender.this.content;
+            HttpContent content = HttpSender.this.content;
             if (content == null)
                 return;
 
@@ -719,7 +745,7 @@ public abstract class HttpSender implements AsyncContentProvider.Listener
             }
             else
             {
-                // Was any content sent while committing ?
+                // Was any content sent while committing?
                 ByteBuffer contentBuffer = content.getContent();
                 if (contentBuffer != null)
                 {
@@ -804,9 +830,9 @@ public abstract class HttpSender implements AsyncContentProvider.Listener
             while (true)
             {
                 boolean advanced = content.advance();
-                boolean consumed = content.isConsumed();
+                boolean lastContent = content.isLast();
                 if (LOG.isDebugEnabled())
-                    LOG.debug("Content {} consumed {} for {}", advanced, consumed, exchange.getRequest());
+                    LOG.debug("Content present {}, last {}, consumed {} for {}", advanced, lastContent, content.isConsumed(), exchange.getRequest());
 
                 if (advanced)
                 {
@@ -814,7 +840,7 @@ public abstract class HttpSender implements AsyncContentProvider.Listener
                     return Action.SCHEDULED;
                 }
 
-                if (consumed)
+                if (lastContent)
                 {
                     sendContent(exchange, content, lastCallback);
                     return Action.IDLE;
@@ -875,12 +901,12 @@ public abstract class HttpSender implements AsyncContentProvider.Listener
         @Override
         protected void onCompleteSuccess()
         {
-            // Nothing to do, since we always return false from process().
-            // Termination is obtained via LastContentCallback.
+            // Nothing to do, since we always return IDLE from process().
+            // Termination is obtained via LastCallback.
         }
     }
 
-    private class LastContentCallback implements Callback
+    private class LastCallback implements Callback
     {
         @Override
         public void succeeded()
