@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2020 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -75,4 +75,12 @@ public interface AuthenticationStore
      * @return the {@link Authentication.Result} that matches the given URI, or null
      */
     Authentication.Result findAuthenticationResult(URI uri);
+
+    /**
+     * @return false if there are no stored authentication results, true if there may be some.
+     */
+    default boolean hasAuthenticationResults()
+    {
+        return true;
+    }
 }
