@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2019 Mort Bay Consulting Pty. Ltd.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -21,27 +21,30 @@ package org.eclipse.jetty.http.pathmap;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
-public class RegexPathSpec extends PathSpec
+public class RegexPathSpec extends AbstractPathSpec
 {
-    protected Pattern pattern;
-
-    protected RegexPathSpec()
-    {
-        super();
-    }
+    private final String _declaration;
+    private final PathSpecGroup _group;
+    private final int _pathDepth;
+    private final int _specLength;
+    private final Pattern _pattern;
 
     public RegexPathSpec(String regex)
     {
-        super.pathSpec = regex;
+        String declaration;
         if (regex.startsWith("regex|"))
-            super.pathSpec = regex.substring("regex|".length());
-        this.pathDepth = 0;
-        this.specLength = pathSpec.length();
+            declaration = regex.substring("regex|".length());
+        else
+            declaration = regex;
+        int specLength = declaration.length();
         // build up a simple signature we can use to identify the grouping
         boolean inGrouping = false;
         StringBuilder signature = new StringBuilder();
-        for (char c : pathSpec.toCharArray())
+
+        int pathDepth = 0;
+        for (int i = 0; i < declaration.length(); i++)
         {
+            char c = declaration.charAt(i);
             switch (c)
             {
                 case '[':
@@ -56,54 +59,64 @@ public class RegexPathSpec extends PathSpec
                     break;
                 case '/':
                     if (!inGrouping)
-                    {
-                        this.pathDepth++;
-                    }
+                        pathDepth++;
                     break;
                 default:
-                    if (!inGrouping)
-                    {
-                        if (Character.isLetterOrDigit(c))
-                        {
-                            signature.append('l'); // literal (exact)
-                        }
-                    }
+                    if (!inGrouping && Character.isLetterOrDigit(c))
+                        signature.append('l'); // literal (exact)
                     break;
             }
         }
-        this.pattern = Pattern.compile(pathSpec);
+        Pattern pattern = Pattern.compile(declaration);
 
         // Figure out the grouping based on the signature
         String sig = signature.toString();
 
-        if (Pattern.matches("^l*$",sig))
-        {
-            this.group = PathSpecGroup.EXACT;
-        }
-        else if (Pattern.matches("^l*g+",sig))
-        {
-            this.group = PathSpecGroup.PREFIX_GLOB;
-        }
-        else if (Pattern.matches("^g+l+$",sig))
-        {
-            this.group = PathSpecGroup.SUFFIX_GLOB;
-        }
+        PathSpecGroup group;
+        if (Pattern.matches("^l*$", sig))
+            group = PathSpecGroup.EXACT;
+        else if (Pattern.matches("^l*g+", sig))
+            group = PathSpecGroup.PREFIX_GLOB;
+        else if (Pattern.matches("^g+l+$", sig))
+            group = PathSpecGroup.SUFFIX_GLOB;
         else
-        {
-            this.group = PathSpecGroup.MIDDLE_GLOB;
-        }
+            group = PathSpecGroup.MIDDLE_GLOB;
+
+        _declaration = declaration;
+        _group = group;
+        _pathDepth = pathDepth;
+        _specLength = specLength;
+        _pattern = pattern;
     }
 
-    public Matcher getMatcher(String path)
+    protected Matcher getMatcher(String path)
     {
-        return this.pattern.matcher(path);
+        return _pattern.matcher(path);
+    }
+
+    @Override
+    public int getSpecLength()
+    {
+        return _specLength;
+    }
+
+    @Override
+    public PathSpecGroup getGroup()
+    {
+        return _group;
+    }
+
+    @Override
+    public int getPathDepth()
+    {
+        return _pathDepth;
     }
 
     @Override
     public String getPathInfo(String path)
     {
         // Path Info only valid for PREFIX_GLOB types
-        if (group == PathSpecGroup.PREFIX_GLOB)
+        if (_group == PathSpecGroup.PREFIX_GLOB)
         {
             Matcher matcher = getMatcher(path);
             if (matcher.matches())
@@ -112,13 +125,9 @@ public class RegexPathSpec extends PathSpec
                 {
                     String pathInfo = matcher.group(1);
                     if ("".equals(pathInfo))
-                    {
                         return "/";
-                    }
                     else
-                    {
                         return pathInfo;
-                    }
                 }
             }
         }
@@ -137,10 +146,8 @@ public class RegexPathSpec extends PathSpec
                 if (idx > 0)
                 {
                     if (path.charAt(idx - 1) == '/')
-                    {
                         idx--;
-                    }
-                    return path.substring(0,idx);
+                    return path.substring(0, idx);
                 }
             }
             return path;
@@ -148,18 +155,29 @@ public class RegexPathSpec extends PathSpec
         return null;
     }
 
-    public Pattern getPattern()
+    @Override
+    public String getDeclaration()
     {
-        return this.pattern;
+        return _declaration;
     }
 
     @Override
-    public String getRelativePath(String base, String path)
+    public String getPrefix()
     {
-        // TODO Auto-generated method stub
         return null;
     }
 
+    @Override
+    public String getSuffix()
+    {
+        return null;
+    }
+
+    public Pattern getPattern()
+    {
+        return _pattern;
+    }
+
     @Override
     public boolean matches(final String path)
     {
@@ -167,7 +185,7 @@ public class RegexPathSpec extends PathSpec
         if (idx >= 0)
         {
             // match only non-query part
-            return getMatcher(path.substring(0,idx)).matches();
+            return getMatcher(path.substring(0, idx)).matches();
         }
         else
         {
