@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2020 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -35,13 +35,11 @@ import javax.servlet.http.HttpSession;
 import org.eclipse.jetty.http.HttpHeader;
 import org.eclipse.jetty.http.HttpHeaderValue;
 import org.eclipse.jetty.http.HttpMethod;
-import org.eclipse.jetty.http.HttpVersion;
 import org.eclipse.jetty.http.MimeTypes;
 import org.eclipse.jetty.security.ServerAuthException;
 import org.eclipse.jetty.security.UserAuthentication;
 import org.eclipse.jetty.server.Authentication;
 import org.eclipse.jetty.server.Authentication.User;
-import org.eclipse.jetty.server.HttpChannel;
 import org.eclipse.jetty.server.Request;
 import org.eclipse.jetty.server.Response;
 import org.eclipse.jetty.server.UserIdentity;
@@ -64,22 +62,20 @@ import org.eclipse.jetty.util.security.Constraint;
  * to the /j_security_check URI within the context.  FormAuthentication uses
  * {@link SessionAuthentication} to wrap Authentication results so that they
  * are  associated with the session.</p>
- *
- *
  */
 public class FormAuthenticator extends LoginAuthenticator
 {
     private static final Logger LOG = Log.getLogger(FormAuthenticator.class);
 
-    public final static String __FORM_LOGIN_PAGE="org.eclipse.jetty.security.form_login_page";
-    public final static String __FORM_ERROR_PAGE="org.eclipse.jetty.security.form_error_page";
-    public final static String __FORM_DISPATCH="org.eclipse.jetty.security.dispatch";
-    public final static String __J_URI = "org.eclipse.jetty.security.form_URI";
-    public final static String __J_POST = "org.eclipse.jetty.security.form_POST";
-    public final static String __J_METHOD = "org.eclipse.jetty.security.form_METHOD";
-    public final static String __J_SECURITY_CHECK = "/j_security_check";
-    public final static String __J_USERNAME = "j_username";
-    public final static String __J_PASSWORD = "j_password";
+    public static final String __FORM_LOGIN_PAGE = "org.eclipse.jetty.security.form_login_page";
+    public static final String __FORM_ERROR_PAGE = "org.eclipse.jetty.security.form_error_page";
+    public static final String __FORM_DISPATCH = "org.eclipse.jetty.security.dispatch";
+    public static final String __J_URI = "org.eclipse.jetty.security.form_URI";
+    public static final String __J_POST = "org.eclipse.jetty.security.form_POST";
+    public static final String __J_METHOD = "org.eclipse.jetty.security.form_METHOD";
+    public static final String __J_SECURITY_CHECK = "/j_security_check";
+    public static final String __J_USERNAME = "j_username";
+    public static final String __J_PASSWORD = "j_password";
 
     private String _formErrorPage;
     private String _formErrorPath;
@@ -92,38 +88,34 @@ public class FormAuthenticator extends LoginAuthenticator
     {
     }
 
-    /* ------------------------------------------------------------ */
-    public FormAuthenticator(String login,String error,boolean dispatch)
+    public FormAuthenticator(String login, String error, boolean dispatch)
     {
         this();
-        if (login!=null)
+        if (login != null)
             setLoginPage(login);
-        if (error!=null)
+        if (error != null)
             setErrorPage(error);
-        _dispatch=dispatch;
+        _dispatch = dispatch;
     }
 
-    /* ------------------------------------------------------------ */
     /**
      * If true, uris that cause a redirect to a login page will always
      * be remembered. If false, only the first uri that leads to a login
      * page redirect is remembered.
      * See https://bugs.eclipse.org/bugs/show_bug.cgi?id=379909
-     * @param alwaysSave
+     *
+     * @param alwaysSave true to always save the uri
      */
-    public void setAlwaysSaveUri (boolean alwaysSave)
+    public void setAlwaysSaveUri(boolean alwaysSave)
     {
         _alwaysSaveUri = alwaysSave;
     }
 
-
-    /* ------------------------------------------------------------ */
-    public boolean getAlwaysSaveUri ()
+    public boolean getAlwaysSaveUri()
     {
         return _alwaysSaveUri;
     }
 
-    /* ------------------------------------------------------------ */
     /**
      * @see org.eclipse.jetty.security.authentication.LoginAuthenticator#setConfiguration(org.eclipse.jetty.security.Authenticator.AuthConfiguration)
      */
@@ -131,24 +123,22 @@ public class FormAuthenticator extends LoginAuthenticator
     public void setConfiguration(AuthConfiguration configuration)
     {
         super.setConfiguration(configuration);
-        String login=configuration.getInitParameter(FormAuthenticator.__FORM_LOGIN_PAGE);
-        if (login!=null)
+        String login = configuration.getInitParameter(FormAuthenticator.__FORM_LOGIN_PAGE);
+        if (login != null)
             setLoginPage(login);
-        String error=configuration.getInitParameter(FormAuthenticator.__FORM_ERROR_PAGE);
-        if (error!=null)
+        String error = configuration.getInitParameter(FormAuthenticator.__FORM_ERROR_PAGE);
+        if (error != null)
             setErrorPage(error);
-        String dispatch=configuration.getInitParameter(FormAuthenticator.__FORM_DISPATCH);
-        _dispatch = dispatch==null?_dispatch:Boolean.valueOf(dispatch);
+        String dispatch = configuration.getInitParameter(FormAuthenticator.__FORM_DISPATCH);
+        _dispatch = dispatch == null ? _dispatch : Boolean.parseBoolean(dispatch);
     }
 
-    /* ------------------------------------------------------------ */
     @Override
     public String getAuthMethod()
     {
         return Constraint.__FORM_AUTH;
     }
 
-    /* ------------------------------------------------------------ */
     private void setLoginPage(String path)
     {
         if (!path.startsWith("/"))
@@ -162,7 +152,6 @@ public class FormAuthenticator extends LoginAuthenticator
             _formLoginPath = _formLoginPath.substring(0, _formLoginPath.indexOf('?'));
     }
 
-    /* ------------------------------------------------------------ */
     private void setErrorPage(String path)
     {
         if (path == null || path.trim().length() == 0)
@@ -184,25 +173,36 @@ public class FormAuthenticator extends LoginAuthenticator
                 _formErrorPath = _formErrorPath.substring(0, _formErrorPath.indexOf('?'));
         }
     }
-    
-    
-    /* ------------------------------------------------------------ */
+
     @Override
     public UserIdentity login(String username, Object password, ServletRequest request)
     {
-        
-        UserIdentity user = super.login(username,password,request);
-        if (user!=null)
+
+        UserIdentity user = super.login(username, password, request);
+        if (user != null)
         {
+
             HttpSession session = ((HttpServletRequest)request).getSession(true);
-            Authentication cached=new SessionAuthentication(getAuthMethod(),user,password);
+            Authentication cached = new SessionAuthentication(getAuthMethod(), user, password);
             session.setAttribute(SessionAuthentication.__J_AUTHENTICATED, cached);
         }
         return user;
     }
-    
-    
-    /* ------------------------------------------------------------ */
+
+    @Override
+    public void logout(ServletRequest request)
+    {
+        super.logout(request);
+        HttpServletRequest httpRequest = (HttpServletRequest)request;
+        HttpSession session = httpRequest.getSession(false);
+
+        if (session == null)
+            return;
+
+        //clean up session
+        session.removeAttribute(SessionAuthentication.__J_AUTHENTICATED);
+    }
+
     @Override
     public void prepareRequest(ServletRequest request)
     {
@@ -217,62 +217,48 @@ public class FormAuthenticator extends LoginAuthenticator
         HttpSession session = httpRequest.getSession(false);
         if (session == null || session.getAttribute(SessionAuthentication.__J_AUTHENTICATED) == null)
             return; //not authenticated yet
-        
+
         String juri = (String)session.getAttribute(__J_URI);
         if (juri == null || juri.length() == 0)
             return; //no original uri saved
-        
+
         String method = (String)session.getAttribute(__J_METHOD);
         if (method == null || method.length() == 0)
             return; //didn't save original request method
-       
+
         StringBuffer buf = httpRequest.getRequestURL();
         if (httpRequest.getQueryString() != null)
             buf.append("?").append(httpRequest.getQueryString());
-        
+
         if (!juri.equals(buf.toString()))
             return; //this request is not for the same url as the original
-        
+
         //restore the original request's method on this request
-        if (LOG.isDebugEnabled()) LOG.debug("Restoring original method {} for {} with method {}", method, juri,httpRequest.getMethod());
-        Request base_request = HttpChannel.getCurrentHttpChannel().getRequest();
-        HttpMethod m = HttpMethod.fromString(method);
-        base_request.setMethod(m,m.asString());
+        if (LOG.isDebugEnabled())
+            LOG.debug("Restoring original method {} for {} with method {}", method, juri, httpRequest.getMethod());
+        Request baseRequest = Request.getBaseRequest(request);
+        baseRequest.setMethod(method);
     }
 
-    /* ------------------------------------------------------------ */
     @Override
     public Authentication validateRequest(ServletRequest req, ServletResponse res, boolean mandatory) throws ServerAuthException
     {
         HttpServletRequest request = (HttpServletRequest)req;
         HttpServletResponse response = (HttpServletResponse)res;
-        String uri = request.getRequestURI();
-        if (uri==null)
-            uri=URIUtil.SLASH;
+        Request baseRequest = Request.getBaseRequest(request);
+        Response baseResponse = baseRequest.getResponse();
 
-        mandatory|=isJSecurityCheck(uri);
+        String uri = request.getRequestURI();
+        if (uri == null)
+            uri = URIUtil.SLASH;
+
+        mandatory |= isJSecurityCheck(uri);
         if (!mandatory)
             return new DeferredAuthentication(this);
 
-        if (isLoginOrErrorPage(URIUtil.addPaths(request.getServletPath(),request.getPathInfo())) &&!DeferredAuthentication.isDeferred(response))
+        if (isLoginOrErrorPage(URIUtil.addPaths(request.getServletPath(), request.getPathInfo())) && !DeferredAuthentication.isDeferred(response))
             return new DeferredAuthentication(this);
 
-        HttpSession session = null;
-        try
-        {
-            session = request.getSession(true);
-        }
-        catch (Exception e)
-        {
-            if (LOG.isDebugEnabled())
-                LOG.debug(e);
-        }
-        
-        //if unable to create a session, user must be
-        //unauthenticated
-        if (session == null)
-            return Authentication.UNAUTHENTICATED;
-
         try
         {
             // Handle a request for authentication.
@@ -282,16 +268,16 @@ public class FormAuthenticator extends LoginAuthenticator
                 final String password = request.getParameter(__J_PASSWORD);
 
                 UserIdentity user = login(username, password, request);
-                LOG.debug("jsecuritycheck {} {}",username,user);
-                session = request.getSession(true);
-                if (user!=null)
-                {                    
+                LOG.debug("jsecuritycheck {} {}", username, user);
+                HttpSession session = request.getSession(false);
+                if (user != null)
+                {
                     // Redirect to original request
                     String nuri;
-                    FormAuthentication form_auth;
-                    synchronized(session)
+                    FormAuthentication formAuth;
+                    synchronized (session)
                     {
-                        nuri = (String) session.getAttribute(__J_URI);
+                        nuri = (String)session.getAttribute(__J_URI);
 
                         if (nuri == null || nuri.length() == 0)
                         {
@@ -299,16 +285,13 @@ public class FormAuthenticator extends LoginAuthenticator
                             if (nuri.length() == 0)
                                 nuri = URIUtil.SLASH;
                         }
-                        form_auth = new FormAuthentication(getAuthMethod(),user);
+                        formAuth = new FormAuthentication(getAuthMethod(), user);
                     }
-                    LOG.debug("authenticated {}->{}",form_auth,nuri);
+                    LOG.debug("authenticated {}->{}", formAuth, nuri);
 
                     response.setContentLength(0);
-                    Response base_response = HttpChannel.getCurrentHttpChannel().getResponse();
-                    Request base_request = HttpChannel.getCurrentHttpChannel().getRequest();
-                    int redirectCode = (base_request.getHttpVersion().getVersion() < HttpVersion.HTTP_1_1.getVersion() ? HttpServletResponse.SC_MOVED_TEMPORARILY : HttpServletResponse.SC_SEE_OTHER);
-                    base_response.sendRedirect(redirectCode, response.encodeRedirectURL(nuri));
-                    return form_auth;
+                    baseResponse.sendRedirect(response.encodeRedirectURL(nuri), true);
+                    return formAuth;
                 }
 
                 // not authenticated
@@ -316,64 +299,61 @@ public class FormAuthenticator extends LoginAuthenticator
                     LOG.debug("Form authentication FAILED for " + StringUtil.printable(username));
                 if (_formErrorPage == null)
                 {
-                    LOG.debug("auth failed {}->403",username);
+                    LOG.debug("auth failed {}->403", username);
                     if (response != null)
                         response.sendError(HttpServletResponse.SC_FORBIDDEN);
                 }
                 else if (_dispatch)
                 {
-                    LOG.debug("auth failed {}=={}",username,_formErrorPage);
+                    LOG.debug("auth failed {}=={}", username, _formErrorPage);
                     RequestDispatcher dispatcher = request.getRequestDispatcher(_formErrorPage);
-                    response.setHeader(HttpHeader.CACHE_CONTROL.asString(),HttpHeaderValue.NO_CACHE.asString());
-                    response.setDateHeader(HttpHeader.EXPIRES.asString(),1);
+                    response.setHeader(HttpHeader.CACHE_CONTROL.asString(), HttpHeaderValue.NO_CACHE.asString());
+                    response.setDateHeader(HttpHeader.EXPIRES.asString(), 1);
                     dispatcher.forward(new FormRequest(request), new FormResponse(response));
                 }
                 else
                 {
-                    LOG.debug("auth failed {}->{}",username,_formErrorPage);
-                    Response base_response = HttpChannel.getCurrentHttpChannel().getResponse();
-                    Request base_request = HttpChannel.getCurrentHttpChannel().getRequest();
-                    int redirectCode = (base_request.getHttpVersion().getVersion() < HttpVersion.HTTP_1_1.getVersion() ? HttpServletResponse.SC_MOVED_TEMPORARILY : HttpServletResponse.SC_SEE_OTHER);
-                    base_response.sendRedirect(redirectCode, response.encodeRedirectURL(URIUtil.addPaths(request.getContextPath(),_formErrorPage)));
+                    LOG.debug("auth failed {}->{}", username, _formErrorPage);
+                    baseResponse.sendRedirect(response.encodeRedirectURL(URIUtil.addPaths(request.getContextPath(), _formErrorPage)), true);
                 }
 
                 return Authentication.SEND_FAILURE;
             }
 
             // Look for cached authentication
-            Authentication authentication = (Authentication) session.getAttribute(SessionAuthentication.__J_AUTHENTICATED);
+            HttpSession session = request.getSession(false);
+            Authentication authentication = session == null ? null : (Authentication)session.getAttribute(SessionAuthentication.__J_AUTHENTICATED);
             if (authentication != null)
             {
                 // Has authentication been revoked?
                 if (authentication instanceof Authentication.User &&
-                    _loginService!=null &&
+                    _loginService != null &&
                     !_loginService.validate(((Authentication.User)authentication).getUserIdentity()))
                 {
-                    LOG.debug("auth revoked {}",authentication);
+                    LOG.debug("auth revoked {}", authentication);
                     session.removeAttribute(SessionAuthentication.__J_AUTHENTICATED);
                 }
                 else
                 {
                     synchronized (session)
                     {
-                        String j_uri=(String)session.getAttribute(__J_URI);
-                        if (j_uri!=null)
+                        String jUri = (String)session.getAttribute(__J_URI);
+                        if (jUri != null)
                         {
                             //check if the request is for the same url as the original and restore
                             //params if it was a post
-                            LOG.debug("auth retry {}->{}",authentication,j_uri);
+                            LOG.debug("auth retry {}->{}", authentication, jUri);
                             StringBuffer buf = request.getRequestURL();
                             if (request.getQueryString() != null)
                                 buf.append("?").append(request.getQueryString());
 
-                            if (j_uri.equals(buf.toString()))
+                            if (jUri.equals(buf.toString()))
                             {
-                                MultiMap<String> j_post = (MultiMap<String>)session.getAttribute(__J_POST);
-                                if (j_post!=null)
+                                MultiMap<String> jPost = (MultiMap<String>)session.getAttribute(__J_POST);
+                                if (jPost != null)
                                 {
-                                    LOG.debug("auth rePOST {}->{}",authentication,j_uri);
-                                    Request base_request = HttpChannel.getCurrentHttpChannel().getRequest();
-                                    base_request.setContentParameters(j_post);
+                                    LOG.debug("auth rePOST {}->{}", authentication, jUri);
+                                    baseRequest.setContentParameters(jPost);
                                 }
                                 session.removeAttribute(__J_URI);
                                 session.removeAttribute(__J_METHOD);
@@ -381,7 +361,7 @@ public class FormAuthenticator extends LoginAuthenticator
                             }
                         }
                     }
-                    LOG.debug("auth {}",authentication);
+                    LOG.debug("auth {}", authentication);
                     return authentication;
                 }
             }
@@ -389,15 +369,16 @@ public class FormAuthenticator extends LoginAuthenticator
             // if we can't send challenge
             if (DeferredAuthentication.isDeferred(response))
             {
-                LOG.debug("auth deferred {}",session.getId());
+                LOG.debug("auth deferred {}", session == null ? null : session.getId());
                 return Authentication.UNAUTHENTICATED;
             }
 
             // remember the current URI
+            session = (session != null ? session : request.getSession(true));
             synchronized (session)
             {
                 // But only if it is not set already, or we save every uri that leads to a login form redirect
-                if (session.getAttribute(__J_URI)==null || _alwaysSaveUri)
+                if (session.getAttribute(__J_URI) == null || _alwaysSaveUri)
                 {
                     StringBuffer buf = request.getRequestURL();
                     if (request.getQueryString() != null)
@@ -407,9 +388,8 @@ public class FormAuthenticator extends LoginAuthenticator
 
                     if (MimeTypes.Type.FORM_ENCODED.is(req.getContentType()) && HttpMethod.POST.is(request.getMethod()))
                     {
-                        Request base_request = (req instanceof Request)?(Request)req:HttpChannel.getCurrentHttpChannel().getRequest();
                         MultiMap<String> formParameters = new MultiMap<>();
-                        base_request.extractFormParameters(formParameters);
+                        baseRequest.extractFormParameters(formParameters);
                         session.setAttribute(__J_POST, formParameters);
                     }
                 }
@@ -418,19 +398,16 @@ public class FormAuthenticator extends LoginAuthenticator
             // send the the challenge
             if (_dispatch)
             {
-                LOG.debug("challenge {}=={}",session.getId(),_formLoginPage);
+                LOG.debug("challenge {}=={}", session.getId(), _formLoginPage);
                 RequestDispatcher dispatcher = request.getRequestDispatcher(_formLoginPage);
-                response.setHeader(HttpHeader.CACHE_CONTROL.asString(),HttpHeaderValue.NO_CACHE.asString());
-                response.setDateHeader(HttpHeader.EXPIRES.asString(),1);
+                response.setHeader(HttpHeader.CACHE_CONTROL.asString(), HttpHeaderValue.NO_CACHE.asString());
+                response.setDateHeader(HttpHeader.EXPIRES.asString(), 1);
                 dispatcher.forward(new FormRequest(request), new FormResponse(response));
             }
             else
             {
-                LOG.debug("challenge {}->{}",session.getId(),_formLoginPage);
-                Response base_response = HttpChannel.getCurrentHttpChannel().getResponse();
-                Request base_request = HttpChannel.getCurrentHttpChannel().getRequest();
-                int redirectCode = (base_request.getHttpVersion().getVersion() < HttpVersion.HTTP_1_1.getVersion() ? HttpServletResponse.SC_MOVED_TEMPORARILY : HttpServletResponse.SC_SEE_OTHER);
-                base_response.sendRedirect(redirectCode, response.encodeRedirectURL(URIUtil.addPaths(request.getContextPath(),_formLoginPage)));
+                LOG.debug("challenge {}->{}", session.getId(), _formLoginPage);
+                baseResponse.sendRedirect(response.encodeRedirectURL(URIUtil.addPaths(request.getContextPath(), _formLoginPage)), true);
             }
             return Authentication.SEND_CONTINUE;
         }
@@ -440,35 +417,30 @@ public class FormAuthenticator extends LoginAuthenticator
         }
     }
 
-    /* ------------------------------------------------------------ */
     public boolean isJSecurityCheck(String uri)
     {
         int jsc = uri.indexOf(__J_SECURITY_CHECK);
 
-        if (jsc<0)
+        if (jsc < 0)
             return false;
-        int e=jsc+__J_SECURITY_CHECK.length();
-        if (e==uri.length())
+        int e = jsc + __J_SECURITY_CHECK.length();
+        if (e == uri.length())
             return true;
         char c = uri.charAt(e);
-        return c==';'||c=='#'||c=='/'||c=='?';
+        return c == ';' || c == '#' || c == '/' || c == '?';
     }
 
-    /* ------------------------------------------------------------ */
     public boolean isLoginOrErrorPage(String pathInContext)
     {
         return pathInContext != null && (pathInContext.equals(_formErrorPath) || pathInContext.equals(_formLoginPath));
     }
 
-    /* ------------------------------------------------------------ */
     @Override
     public boolean secureResponse(ServletRequest req, ServletResponse res, boolean mandatory, User validatedUser) throws ServerAuthException
     {
         return true;
     }
 
-    /* ------------------------------------------------------------ */
-    /* ------------------------------------------------------------ */
     protected static class FormRequest extends HttpServletRequestWrapper
     {
         public FormRequest(HttpServletRequest request)
@@ -502,13 +474,11 @@ public class FormAuthenticator extends LoginAuthenticator
         public Enumeration<String> getHeaders(String name)
         {
             if (name.toLowerCase(Locale.ENGLISH).startsWith("if-"))
-                return Collections.<String>enumeration(Collections.<String>emptyList());
+                return Collections.enumeration(Collections.emptyList());
             return super.getHeaders(name);
         }
     }
 
-    /* ------------------------------------------------------------ */
-    /* ------------------------------------------------------------ */
     protected static class FormResponse extends HttpServletResponseWrapper
     {
         public FormResponse(HttpServletResponse response)
@@ -520,45 +490,43 @@ public class FormAuthenticator extends LoginAuthenticator
         public void addDateHeader(String name, long date)
         {
             if (notIgnored(name))
-                super.addDateHeader(name,date);
+                super.addDateHeader(name, date);
         }
 
         @Override
         public void addHeader(String name, String value)
         {
             if (notIgnored(name))
-                super.addHeader(name,value);
+                super.addHeader(name, value);
         }
 
         @Override
         public void setDateHeader(String name, long date)
         {
             if (notIgnored(name))
-                super.setDateHeader(name,date);
+                super.setDateHeader(name, date);
         }
 
         @Override
         public void setHeader(String name, String value)
         {
             if (notIgnored(name))
-                super.setHeader(name,value);
+                super.setHeader(name, value);
         }
 
         private boolean notIgnored(String name)
         {
-            if (HttpHeader.CACHE_CONTROL.is(name) ||
-                HttpHeader.PRAGMA.is(name) ||
-                HttpHeader.ETAG.is(name) ||
-                HttpHeader.EXPIRES.is(name) ||
-                HttpHeader.LAST_MODIFIED.is(name) ||
-                HttpHeader.AGE.is(name))
-                return false;
-            return true;
+            return !HttpHeader.CACHE_CONTROL.is(name) &&
+                !HttpHeader.PRAGMA.is(name) &&
+                !HttpHeader.ETAG.is(name) &&
+                !HttpHeader.EXPIRES.is(name) &&
+                !HttpHeader.LAST_MODIFIED.is(name) &&
+                !HttpHeader.AGE.is(name);
         }
     }
 
-    /* ------------------------------------------------------------ */
-    /** This Authentication represents a just completed Form authentication.
+    /**
+     * This Authentication represents a just completed Form authentication.
      * Subsequent requests from the same user are authenticated by the presents
      * of a {@link SessionAuthentication} instance in their session.
      */
@@ -566,13 +534,13 @@ public class FormAuthenticator extends LoginAuthenticator
     {
         public FormAuthentication(String method, UserIdentity userIdentity)
         {
-            super(method,userIdentity);
+            super(method, userIdentity);
         }
 
         @Override
         public String toString()
         {
-            return "Form"+super.toString();
+            return "Form" + super.toString();
         }
     }
 }
