@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2020 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -23,10 +23,11 @@ import java.util.Locale;
 import javax.servlet.http.Cookie;
 
 import org.eclipse.jetty.http.HttpHeader;
-import org.eclipse.jetty.http.PathMap;
+import org.eclipse.jetty.http.pathmap.PathMappings;
+import org.eclipse.jetty.server.handler.StatisticsHandler;
 import org.eclipse.jetty.util.DateCache;
 import org.eclipse.jetty.util.annotation.ManagedAttribute;
-import org.eclipse.jetty.util.component.AbstractLifeCycle;
+import org.eclipse.jetty.util.component.ContainerLifeCycle;
 import org.eclipse.jetty.util.log.Log;
 import org.eclipse.jetty.util.log.Logger;
 
@@ -35,24 +36,21 @@ import org.eclipse.jetty.util.log.Logger;
  * Configuration options allow a choice between the standard Common Log Format (as used in the 3 log format) and the
  * Combined Log Format (single log format). This log format can be output by most web servers, and almost all web log
  * analysis software can understand these formats.
+ *
+ * @deprecated use {@link CustomRequestLog} given format string {@link CustomRequestLog#EXTENDED_NCSA_FORMAT} with a {@link RequestLog.Writer}
  */
-public abstract class AbstractNCSARequestLog extends AbstractLifeCycle implements RequestLog
+@Deprecated
+public class AbstractNCSARequestLog extends ContainerLifeCycle implements RequestLog
 {
     protected static final Logger LOG = Log.getLogger(AbstractNCSARequestLog.class);
 
-    private static ThreadLocal<StringBuilder> _buffers = new ThreadLocal<StringBuilder>()
-    {
-        @Override
-        protected StringBuilder initialValue()
-        {
-            return new StringBuilder(256);
-        }
-    };
+    private static ThreadLocal<StringBuilder> _buffers = ThreadLocal.withInitial(() -> new StringBuilder(256));
 
+    protected final RequestLog.Writer _requestLogWriter;
 
     private String[] _ignorePaths;
     private boolean _extended;
-    private transient PathMap<String> _ignorePathMap;
+    private transient PathMappings<String> _ignorePathMap;
     private boolean _logLatency = false;
     private boolean _logCookies = false;
     private boolean _logServer = false;
@@ -62,27 +60,45 @@ public abstract class AbstractNCSARequestLog extends AbstractLifeCycle implement
     private Locale _logLocale = Locale.getDefault();
     private String _logTimeZone = "GMT";
 
-    /* ------------------------------------------------------------ */
+    public AbstractNCSARequestLog(RequestLog.Writer requestLogWriter)
+    {
+        this._requestLogWriter = requestLogWriter;
+        addBean(_requestLogWriter);
+    }
 
     /**
      * Is logging enabled
+     *
+     * @return true if logging is enabled
      */
-    protected abstract boolean isEnabled();
-    
-    /* ------------------------------------------------------------ */
+    protected boolean isEnabled()
+    {
+        return true;
+    }
 
     /**
      * Write requestEntry out. (to disk or slf4j log)
+     *
+     * @param requestEntry the request entry
+     * @throws IOException if unable to write the entry
      */
-    public abstract void write(String requestEntry) throws IOException;
+    public void write(String requestEntry) throws IOException
+    {
+        _requestLogWriter.write(requestEntry);
+    }
 
-    /* ------------------------------------------------------------ */
+    private void append(StringBuilder buf, String s)
+    {
+        if (s == null || s.length() == 0)
+            buf.append('-');
+        else
+            buf.append(s);
+    }
 
     /**
      * Writes the request and response information to the output stream.
      *
-     * @see org.eclipse.jetty.server.RequestLog#log(org.eclipse.jetty.server.Request,
-     *      org.eclipse.jetty.server.Response)
+     * @see org.eclipse.jetty.server.RequestLog#log(Request, Response)
      */
     @Override
     public void log(Request request, Response response)
@@ -100,7 +116,7 @@ public abstract class AbstractNCSARequestLog extends AbstractLifeCycle implement
 
             if (_logServer)
             {
-                buf.append(request.getServerName());
+                append(buf, request.getServerName());
                 buf.append(' ');
             }
 
@@ -115,11 +131,9 @@ public abstract class AbstractNCSARequestLog extends AbstractLifeCycle implement
 
             buf.append(addr);
             buf.append(" - ");
-            Authentication authentication = request.getAuthentication();
-            if (authentication instanceof Authentication.User)
-                buf.append(((Authentication.User)authentication).getUserIdentity().getUserPrincipal().getName());
-            else
-                buf.append("-");
+
+            String auth = getAuthentication(request);
+            append(buf, auth == null ? "-" : auth);
 
             buf.append(" [");
             if (_logDateCache != null)
@@ -128,46 +142,48 @@ public abstract class AbstractNCSARequestLog extends AbstractLifeCycle implement
                 buf.append(request.getTimeStamp());
 
             buf.append("] \"");
-            buf.append(request.getMethod());
+            append(buf, request.getMethod());
             buf.append(' ');
-            buf.append(request.getUri().toString());
+            append(buf, request.getOriginalURI());
             buf.append(' ');
-            buf.append(request.getProtocol());
+            append(buf, request.getProtocol());
             buf.append("\" ");
 
-            int status = response.getStatus();
-            if (status <= 0)
-                status = 404;
-            buf.append((char)('0' + ((status / 100) % 10)));
-            buf.append((char)('0' + ((status / 10) % 10)));
-            buf.append((char)('0' + (status % 10)));
+            int status = response.getCommittedMetaData().getStatus();
+            if (status >= 0)
+            {
+                buf.append((char)('0' + ((status / 100) % 10)));
+                buf.append((char)('0' + ((status / 10) % 10)));
+                buf.append((char)('0' + (status % 10)));
+            }
+            else
+                buf.append(status);
 
-            long responseLength = response.getLongContentLength();
-            if (responseLength >= 0)
+            long written = response.getHttpChannel().getBytesWritten();
+            if (written >= 0)
             {
                 buf.append(' ');
-                if (responseLength > 99999)
-                    buf.append(responseLength);
+                if (written > 99999)
+                    buf.append(written);
                 else
                 {
-                    if (responseLength > 9999)
-                        buf.append((char)('0' + ((responseLength / 10000) % 10)));
-                    if (responseLength > 999)
-                        buf.append((char)('0' + ((responseLength / 1000) % 10)));
-                    if (responseLength > 99)
-                        buf.append((char)('0' + ((responseLength / 100) % 10)));
-                    if (responseLength > 9)
-                        buf.append((char)('0' + ((responseLength / 10) % 10)));
-                    buf.append((char)('0' + (responseLength) % 10));
+                    if (written > 9999)
+                        buf.append((char)('0' + ((written / 10000) % 10)));
+                    if (written > 999)
+                        buf.append((char)('0' + ((written / 1000) % 10)));
+                    if (written > 99)
+                        buf.append((char)('0' + ((written / 100) % 10)));
+                    if (written > 9)
+                        buf.append((char)('0' + ((written / 10) % 10)));
+                    buf.append((char)('0' + (written) % 10));
                 }
                 buf.append(' ');
             }
             else
                 buf.append(" - ");
 
-
             if (_extended)
-                logExtended(request, response, buf);
+                logExtended(buf, request, response);
 
             if (_logCookies)
             {
@@ -208,20 +224,34 @@ public abstract class AbstractNCSARequestLog extends AbstractLifeCycle implement
             LOG.warn(e);
         }
     }
-    
-    /* ------------------------------------------------------------ */
+
+    /**
+     * Extract the user authentication
+     *
+     * @param request The request to extract from
+     * @return The string to log for authenticated user.
+     */
+    protected String getAuthentication(Request request)
+    {
+        Authentication authentication = request.getAuthentication();
+
+        if (authentication instanceof Authentication.User)
+            return ((Authentication.User)authentication).getUserIdentity().getUserPrincipal().getName();
+
+        // TODO extract the user name if it is Authentication.Deferred and return as '?username'
+
+        return null;
+    }
 
     /**
      * Writes extended request and response information to the output stream.
      *
-     * @param request  request object
+     * @param b StringBuilder to write to
+     * @param request request object
      * @param response response object
-     * @param b        StringBuilder to write to
-     * @throws IOException
+     * @throws IOException if unable to log the extended information
      */
-    protected void logExtended(Request request,
-                               Response response,
-                               StringBuilder b) throws IOException
+    protected void logExtended(StringBuilder b, Request request, Response response) throws IOException
     {
         String referer = request.getHeader(HttpHeader.REFERER.toString());
         if (referer == null)
@@ -235,7 +265,7 @@ public abstract class AbstractNCSARequestLog extends AbstractLifeCycle implement
 
         String agent = request.getHeader(HttpHeader.USER_AGENT.toString());
         if (agent == null)
-            b.append("\"-\" ");
+            b.append("\"-\"");
         else
         {
             b.append('"');
@@ -244,7 +274,6 @@ public abstract class AbstractNCSARequestLog extends AbstractLifeCycle implement
         }
     }
 
-
     /**
      * Set request paths that will not be logged.
      *
@@ -269,7 +298,7 @@ public abstract class AbstractNCSARequestLog extends AbstractLifeCycle implement
      * Controls logging of the request cookies.
      *
      * @param logCookies true - values of request cookies will be logged, false - values of request cookies will not be
-     *                   logged
+     * logged
      */
     public void setLogCookies(boolean logCookies)
     {
@@ -310,7 +339,7 @@ public abstract class AbstractNCSARequestLog extends AbstractLifeCycle implement
      * Controls logging of request processing time.
      *
      * @param logLatency true - request processing time will be logged false - request processing time will not be
-     *                   logged
+     * logged
      */
     public void setLogLatency(boolean logLatency)
     {
@@ -328,15 +357,19 @@ public abstract class AbstractNCSARequestLog extends AbstractLifeCycle implement
     }
 
     /**
+     * @param value true to log dispatch
      * @deprecated use {@link StatisticsHandler}
      */
+    @Deprecated
     public void setLogDispatch(boolean value)
     {
     }
 
     /**
+     * @return true if logging dispatches
      * @deprecated use {@link StatisticsHandler}
      */
+    @Deprecated
     public boolean isLogDispatch()
     {
         return false;
@@ -347,7 +380,7 @@ public abstract class AbstractNCSARequestLog extends AbstractLifeCycle implement
      * be logged.
      *
      * @param preferProxiedForAddress true - IP address from header will be logged, false - IP address from the
-     *                                connection will be logged
+     * connection will be logged
      */
     public void setPreferProxiedForAddress(boolean preferProxiedForAddress)
     {
@@ -395,14 +428,16 @@ public abstract class AbstractNCSARequestLog extends AbstractLifeCycle implement
     {
         if (_logDateFormat != null)
         {
-            _logDateCache = new DateCache(_logDateFormat, _logLocale ,_logTimeZone);
+            _logDateCache = new DateCache(_logDateFormat, _logLocale, _logTimeZone);
         }
 
         if (_ignorePaths != null && _ignorePaths.length > 0)
         {
-            _ignorePathMap = new PathMap<>();
+            _ignorePathMap = new PathMappings<>();
             for (int i = 0; i < _ignorePaths.length; i++)
+            {
                 _ignorePathMap.put(_ignorePaths[i], _ignorePaths[i]);
+            }
         }
         else
             _ignorePathMap = null;
