@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2019 Mort Bay Consulting Pty. Ltd.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -16,7 +16,6 @@
 //  ========================================================================
 //
 
-
 package org.eclipse.jetty.server;
 
 import java.io.IOException;
@@ -45,12 +44,12 @@ import org.eclipse.jetty.util.log.Log;
 import org.eclipse.jetty.util.log.Logger;
 
 /**
- * A HttpChannel customized to be transported over the HTTP/1 protocol
+ * An HttpChannel customized to be transported over the HTTP/1 protocol
  */
 public class HttpChannelOverHttp extends HttpChannel implements HttpParser.RequestHandler, HttpParser.ComplianceHandler
 {
     private static final Logger LOG = Log.getLogger(HttpChannelOverHttp.class);
-    private final static HttpField PREAMBLE_UPGRADE_H2C = new HttpField(HttpHeader.UPGRADE, "h2c");
+    private static final HttpField PREAMBLE_UPGRADE_H2C = new HttpField(HttpHeader.UPGRADE, "h2c");
     private final HttpFields _fields = new HttpFields();
     private final MetaData.Request _metadata = new MetaData.Request(_fields);
     private final HttpConnection _httpConnection;
@@ -233,7 +232,7 @@ public class HttpChannelOverHttp extends HttpChannel implements HttpParser.Reque
         if (_metadata.getMethod() == null)
             _httpConnection.close();
         else if (onEarlyEOF() || _delayedForContent)
-        { 
+        {
             _delayedForContent = false;
             handle();
         }
@@ -290,7 +289,7 @@ public class HttpChannelOverHttp extends HttpChannel implements HttpParser.Reque
         if (_complianceViolations != null && !_complianceViolations.isEmpty())
         {
             this.getRequest().setAttribute(HttpCompliance.VIOLATIONS_ATTR, _complianceViolations);
-            _complianceViolations=null;
+            _complianceViolations = null;
         }
 
         boolean persistent;
@@ -367,9 +366,9 @@ public class HttpChannelOverHttp extends HttpChannel implements HttpParser.Reque
                 _upgrade = PREAMBLE_UPGRADE_H2C;
 
                 if (HttpMethod.PRI.is(_metadata.getMethod()) &&
-                        "*".equals(_metadata.getURI().toString()) &&
-                        _fields.size() == 0 &&
-                        upgrade())
+                    "*".equals(_metadata.getURI().toString()) &&
+                    _fields.size() == 0 &&
+                    upgrade())
                     return true;
 
                 badMessage(new BadMessageException(HttpStatus.UPGRADE_REQUIRED_426));
@@ -390,11 +389,11 @@ public class HttpChannelOverHttp extends HttpChannel implements HttpParser.Reque
 
         // Should we delay dispatch until we have some content?
         // We should not delay if there is no content expect or client is expecting 100 or the response is already committed or the request buffer already has something in it to parse
-        _delayedForContent = (getHttpConfiguration().isDelayDispatchUntilContent()
-                && (_httpConnection.getParser().getContentLength() > 0 || _httpConnection.getParser().isChunking())
-                && !isExpecting100Continue()
-                && !isCommitted()
-                && _httpConnection.isRequestBufferEmpty());
+        _delayedForContent = (getHttpConfiguration().isDelayDispatchUntilContent() &&
+            (_httpConnection.getParser().getContentLength() > 0 || _httpConnection.getParser().isChunking()) &&
+            !isExpecting100Continue() &&
+            !isCommitted() &&
+            _httpConnection.isRequestBufferEmpty());
 
         return !_delayedForContent;
     }
@@ -412,7 +411,7 @@ public class HttpChannelOverHttp extends HttpChannel implements HttpParser.Reque
     }
 
     /**
-     * <p>Attempts to perform a HTTP/1.1 upgrade.</p>
+     * <p>Attempts to perform an HTTP/1.1 upgrade.</p>
      * <p>The upgrade looks up a {@link ConnectionFactory.Upgrading} from the connector
      * matching the protocol specified in the {@code Upgrade} header.</p>
      * <p>The upgrade may succeed, be ignored (which can allow a later handler to implement)
@@ -427,9 +426,9 @@ public class HttpChannelOverHttp extends HttpChannel implements HttpParser.Reque
             LOG.debug("upgrade {} {}", this, _upgrade);
 
         @SuppressWarnings("ReferenceEquality")
-        boolean isUpgraded_H2C = (_upgrade == PREAMBLE_UPGRADE_H2C);
+        boolean isUpgradedH2C = (_upgrade == PREAMBLE_UPGRADE_H2C);
 
-        if (!isUpgraded_H2C && (_connection == null || !_connection.contains("upgrade")))
+        if (!isUpgradedH2C && (_connection == null || !_connection.contains("upgrade")))
             throw new BadMessageException(HttpStatus.BAD_REQUEST_400);
 
         // Find the upgrade factory
@@ -455,8 +454,8 @@ public class HttpChannelOverHttp extends HttpChannel implements HttpParser.Reque
 
         // Create new connection
         HttpFields response101 = new HttpFields();
-        Connection upgrade_connection = factory.upgradeConnection(getConnector(), getEndPoint(), _metadata, response101);
-        if (upgrade_connection == null)
+        Connection upgradeConnection = factory.upgradeConnection(getConnector(), getEndPoint(), _metadata, response101);
+        if (upgradeConnection == null)
         {
             if (LOG.isDebugEnabled())
                 LOG.debug("Upgrade ignored for {} by {}", _upgrade, factory);
@@ -466,7 +465,7 @@ public class HttpChannelOverHttp extends HttpChannel implements HttpParser.Reque
         // Send 101 if needed
         try
         {
-            if (!isUpgraded_H2C)
+            if (!isUpgradedH2C)
                 sendResponse(new MetaData.Response(HttpVersion.HTTP_1_1, HttpStatus.SWITCHING_PROTOCOLS_101, response101, 0), null, true);
         }
         catch (IOException e)
@@ -475,8 +474,8 @@ public class HttpChannelOverHttp extends HttpChannel implements HttpParser.Reque
         }
 
         if (LOG.isDebugEnabled())
-            LOG.debug("Upgrade from {} to {}", getEndPoint().getConnection(), upgrade_connection);
-        getRequest().setAttribute(HttpConnection.UPGRADE_CONNECTION_ATTRIBUTE, upgrade_connection);
+            LOG.debug("Upgrade from {} to {}", getEndPoint().getConnection(), upgradeConnection);
+        getRequest().setAttribute(HttpConnection.UPGRADE_CONNECTION_ATTRIBUTE, upgradeConnection);
         getResponse().setStatus(101);
         getHttpTransport().onCompleted();
         return true;
@@ -527,7 +526,7 @@ public class HttpChannelOverHttp extends HttpChannel implements HttpParser.Reque
             {
                 _complianceViolations = new ArrayList<>();
             }
-            String record = String.format("%s (see %s) in mode %s for %s in %s", 
+            String record = String.format("%s (see %s) in mode %s for %s in %s",
                 violation.getDescription(), violation.getURL(), compliance, reason, getHttpTransport());
             _complianceViolations.add(record);
             if (LOG.isDebugEnabled())
