@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2021 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -27,7 +27,6 @@ import java.io.InputStreamReader;
 import java.io.UnsupportedEncodingException;
 import java.net.InetAddress;
 import java.net.InetSocketAddress;
-import java.net.UnknownHostException;
 import java.nio.charset.Charset;
 import java.nio.charset.StandardCharsets;
 import java.nio.charset.UnsupportedCharsetException;
@@ -66,7 +65,6 @@ import javax.servlet.http.Part;
 import org.eclipse.jetty.http.BadMessageException;
 import org.eclipse.jetty.http.HostPortHttpField;
 import org.eclipse.jetty.http.HttpCompliance;
-import org.eclipse.jetty.http.HttpComplianceSection;
 import org.eclipse.jetty.http.HttpCookie;
 import org.eclipse.jetty.http.HttpField;
 import org.eclipse.jetty.http.HttpFields;
@@ -374,10 +372,10 @@ public class Request implements HttpServletRequest
         if (listener instanceof AsyncListener)
             throw new IllegalArgumentException(listener.getClass().toString());
     }
-    
+
     /**
      * Remember a session that this request has just entered.
-     * 
+     *
      * @param s the session
      */
     public void enterSession(HttpSession s)
@@ -401,20 +399,34 @@ public class Request implements HttpServletRequest
     {
         if (LOG.isDebugEnabled())
             LOG.debug("Request {} leaving session {}", this, session);
-        session.getSessionHandler().complete(session);
+        //try and scope to a request and context before leaving the session
+        ServletContext ctx = session.getServletContext();
+        ContextHandler handler = ContextHandler.getContextHandler(ctx);
+        if (handler == null)
+            session.getSessionHandler().complete(session);
+        else
+            handler.handle(this, () ->  session.getSessionHandler().complete(session));
     }
 
     /**
-     * A response is being committed for a session, 
+     * A response is being committed for a session,
      * potentially write the session out before the
      * client receives the response.
+     *
      * @param session the session
      */
     private void commitSession(Session session)
     {
         if (LOG.isDebugEnabled())
             LOG.debug("Response {} committing for session {}", this, session);
-        session.getSessionHandler().commit(session);
+        
+        //try and scope to a request and context before committing the session
+        ServletContext ctx = session.getServletContext();
+        ContextHandler handler = ContextHandler.getContextHandler(ctx);
+        if (handler == null)
+            session.getSessionHandler().commit(session);
+        else
+            handler.handle(this, () -> session.getSessionHandler().commit(session));
     }
 
     private MultiMap<String> getParameters()
@@ -436,6 +448,7 @@ public class Request implements HttpServletRequest
                 }
                 catch (IllegalStateException | IllegalArgumentException e)
                 {
+                    LOG.warn(e.toString());
                     throw new BadMessageException("Unable to parse form content", e);
                 }
             }
@@ -991,30 +1004,20 @@ public class Request implements HttpServletRequest
     @Override
     public String getLocalAddr()
     {
-        if (_channel == null)
+        if (_channel != null)
         {
-            try
-            {
-                String name = InetAddress.getLocalHost().getHostAddress();
-                if (StringUtil.ALL_INTERFACES.equals(name))
-                    return null;
-                return HostPort.normalizeHost(name);
-            }
-            catch (UnknownHostException e)
-            {
-                LOG.ignore(e);
-                return null;
-            }
+            InetSocketAddress local = _channel.getLocalAddress();
+            if (local == null)
+                return "";
+            InetAddress address = local.getAddress();
+            String result = address == null
+                ? local.getHostString()
+                : address.getHostAddress();
+
+            return formatAddrOrHost(result);
         }
 
-        InetSocketAddress local = _channel.getLocalAddress();
-        if (local == null)
-            return "";
-        InetAddress address = local.getAddress();
-        String result = address == null
-            ? local.getHostString()
-            : address.getHostAddress();
-        return HostPort.normalizeHost(result);
+        return "";
     }
 
     /*
@@ -1025,23 +1028,11 @@ public class Request implements HttpServletRequest
     {
         if (_channel != null)
         {
-            InetSocketAddress local = _channel.getLocalAddress();
-            if (local != null)
-                return HostPort.normalizeHost(local.getHostString());
+            String localName = _channel.getLocalName();
+            return formatAddrOrHost(localName);
         }
 
-        try
-        {
-            String name = InetAddress.getLocalHost().getHostName();
-            if (StringUtil.ALL_INTERFACES.equals(name))
-                return null;
-            return HostPort.normalizeHost(name);
-        }
-        catch (UnknownHostException e)
-        {
-            LOG.ignore(e);
-        }
-        return null;
+        return ""; // not allowed to be null
     }
 
     /*
@@ -1050,10 +1041,13 @@ public class Request implements HttpServletRequest
     @Override
     public int getLocalPort()
     {
-        if (_channel == null)
-            return 0;
-        InetSocketAddress local = _channel.getLocalAddress();
-        return local == null ? 0 : local.getPort();
+        if (_channel != null)
+        {
+            int localPort = _channel.getLocalPort();
+            if (localPort > 0)
+                return localPort;
+        }
+        return 0;
     }
 
     /*
@@ -1257,12 +1251,10 @@ public class Request implements HttpServletRequest
 
         InetAddress address = remote.getAddress();
         String result = address == null
-            ? remote.getHostString()
-            : address.getHostAddress();
-        // Add IPv6 brackets if necessary, to be consistent
-        // with cases where _remote has been built from other
-        // sources such as forward headers or PROXY protocol.
-        return HostPort.normalizeHost(result);
+                ? remote.getHostString()
+                : address.getHostAddress();
+
+        return formatAddrOrHost(result);
     }
 
     /*
@@ -1276,8 +1268,9 @@ public class Request implements HttpServletRequest
             remote = _channel.getRemoteAddress();
         if (remote == null)
             return "";
+
         // We want the URI host, so add IPv6 brackets if necessary.
-        return HostPort.normalizeHost(remote.getHostString());
+        return formatAddrOrHost(remote.getHostString());
     }
 
     /*
@@ -1403,7 +1396,7 @@ public class Request implements HttpServletRequest
     public String getServerName()
     {
         MetaData.Request metadata = _metaData;
-        String name = metadata == null ? null : metadata.getURI().getHost();
+        String name = metadata == null ? null : formatAddrOrHost(metadata.getURI().getHost());
 
         // Return already determined host
         if (name != null)
@@ -1425,25 +1418,23 @@ public class Request implements HttpServletRequest
             {
                 HostPortHttpField authority = (HostPortHttpField)host;
                 metadata.getURI().setAuthority(authority.getHost(), authority.getPort());
-                return authority.getHost();
+                return formatAddrOrHost(authority.getHost());
             }
         }
 
+        if (_channel != null)
+        {
+            HostPort serverAuthority = _channel.getServerAuthority();
+            if (serverAuthority != null)
+                return formatAddrOrHost(serverAuthority.getHost());
+        }
+
         // Return host from connection
         String name = getLocalName();
         if (name != null)
-            return HostPort.normalizeHost(name);
+            return formatAddrOrHost(name);
 
-        // Return the local host
-        try
-        {
-            return HostPort.normalizeHost(InetAddress.getLocalHost().getHostAddress());
-        }
-        catch (UnknownHostException e)
-        {
-            LOG.ignore(e);
-        }
-        return null;
+        return ""; // not allowed to be null
     }
 
     /*
@@ -1459,9 +1450,10 @@ public class Request implements HttpServletRequest
         // If no port specified, return the default port for the scheme
         if (port <= 0)
         {
-            if (getScheme().equalsIgnoreCase(URIUtil.HTTPS))
+            if (HttpScheme.HTTPS.is(getScheme()))
                 return 443;
-            return 80;
+            else
+                return 80;
         }
 
         // return a specific port
@@ -1475,19 +1467,25 @@ public class Request implements HttpServletRequest
         HttpField host = metadata == null ? null : metadata.getFields().getField(HttpHeader.HOST);
         if (host != null)
         {
-            // TODO is this needed now?
             HostPortHttpField authority = (host instanceof HostPortHttpField)
                 ? ((HostPortHttpField)host)
                 : new HostPortHttpField(host.getValue());
-            metadata.getURI().setAuthority(authority.getHost(), authority.getPort());
-            return authority.getPort();
+            if (authority.getHostPort().hasHost() && authority.getHostPort().hasPort())
+            {
+                metadata.getURI().setAuthority(authority.getHost(), authority.getPort());
+                return authority.getPort();
+            }
+        }
+
+        if (_channel != null)
+        {
+            HostPort serverAuthority = _channel.getServerAuthority();
+            if (serverAuthority != null)
+                return serverAuthority.getPort();
         }
 
         // Return host from connection
-        if (_channel != null)
-            return getLocalPort();
-
-        return -1;
+        return getLocalPort();
     }
 
     @Override
@@ -1552,7 +1550,7 @@ public class Request implements HttpServletRequest
                 leaveSession(s);
         }
     }
-    
+
     /**
      * Called when a response is about to be committed, ie sent
      * back to the client
@@ -1562,13 +1560,15 @@ public class Request implements HttpServletRequest
         if (_sessions != null)
         {
             for (Session s:_sessions)
+            {
                 commitSession(s);
+            }
         }
     }
 
     /**
      * Find a session that this request has already entered for the
-     * given SessionHandler 
+     * given SessionHandler
      *
      * @param sessionHandler the SessionHandler (ie context) to check
      * @return
@@ -1577,9 +1577,9 @@ public class Request implements HttpServletRequest
     {
         if (_sessions == null || _sessions.size() == 0 || sessionHandler == null)
             return null;
-        
+
         HttpSession session = null;
-        
+
         for (HttpSession s:_sessions)
         {
             Session ss =  Session.class.cast(s);
@@ -1592,7 +1592,7 @@ public class Request implements HttpServletRequest
         }
         return session;
     }
-    
+
     /*
      * @see javax.servlet.http.HttpServletRequest#getSession()
      */
@@ -1628,7 +1628,7 @@ public class Request implements HttpServletRequest
         _session = _sessionHandler.newHttpSession(this);
         if (_session == null)
             throw new IllegalStateException("Create session failed");
-        
+
         HttpCookie cookie = _sessionHandler.getSessionCookie(_session, getContextPath(), isSecure());
         if (cookie != null)
             _channel.getResponse().replaceCookie(cookie);
@@ -1823,21 +1823,17 @@ public class Request implements HttpServletRequest
 
         setMethod(request.getMethod());
         HttpURI uri = request.getURI();
-
-        if (uri.isAmbiguous())
+        if (uri.hasViolations())
         {
-            // replaced in jetty-10 with URICompliance from the HttpConfiguration
+            // Replaced in jetty-10 with URICompliance from the HttpConfiguration.
             Connection connection = _channel == null ? null : _channel.getConnection();
             HttpCompliance compliance = connection instanceof HttpConnection
                 ? ((HttpConnection)connection).getHttpCompliance()
                 : _channel != null ? _channel.getConnector().getBean(HttpCompliance.class) : null;
 
-            if (uri.hasAmbiguousSegment() && (compliance == null || compliance.sections().contains(HttpComplianceSection.NO_AMBIGUOUS_PATH_SEGMENTS)))
-                throw new BadMessageException("Ambiguous segment in URI");
-            if (uri.hasAmbiguousSeparator() && (compliance == null || compliance.sections().contains(HttpComplianceSection.NO_AMBIGUOUS_PATH_SEPARATORS)))
-                throw new BadMessageException("Ambiguous separator in URI");
-            if (uri.hasAmbiguousParameter() && (compliance == null || compliance.sections().contains(HttpComplianceSection.NO_AMBIGUOUS_PATH_PARAMETERS)))
-                throw new BadMessageException("Ambiguous path parameter in URI");
+            String badMessage = HttpCompliance.checkUriCompliance(compliance, uri);
+            if (badMessage != null)
+                throw new BadMessageException(badMessage);
         }
 
         _originalURI = uri.isAbsolute() && request.getHttpVersion() != HttpVersion.HTTP_2 ? uri.toString() : uri.getPathQuery();
@@ -2537,7 +2533,14 @@ public class Request implements HttpServletRequest
         if (newQuery != null)
         {
             newQueryParams = new MultiMap<>();
-            UrlEncoded.decodeTo(newQuery, newQueryParams, UrlEncoded.ENCODING);
+            try
+            {
+                UrlEncoded.decodeTo(newQuery, newQueryParams, UrlEncoded.ENCODING);
+            }
+            catch (Exception e)
+            {
+                throw new BadMessageException(500, "Bad query encoding", e);
+            }
         }
 
         MultiMap<String> oldQueryParams = _queryParameters;
@@ -2608,4 +2611,9 @@ public class Request implements HttpServletRequest
     {
         throw new ServletException("HttpServletRequest.upgrade() not supported in Jetty");
     }
+
+    private String formatAddrOrHost(String name)
+    {
+        return _channel == null ? HostPort.normalizeHost(name) : _channel.formatAddrOrHost(name);
+    }
 }
