@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2021 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -240,7 +240,7 @@ public class ResourceService
             // Find the content
             content = _contentFactory.getContent(pathInContext, response.getBufferSize());
             if (LOG.isDebugEnabled())
-                LOG.info("content={}", content);
+                LOG.debug("content={}", content);
 
             // Not found?
             if (content == null || !content.getResource().exists())
@@ -380,23 +380,20 @@ public class ResourceService
         // Redirect to directory
         if (!endsWithSlash)
         {
-            StringBuffer buf = request.getRequestURL();
-            synchronized (buf)
+            StringBuilder buf = new StringBuilder(request.getRequestURI());
+            int param = buf.lastIndexOf(";");
+            if (param < 0 || buf.lastIndexOf("/", param) > 0)
+                buf.append('/');
+            else
+                buf.insert(param, '/');
+            String q = request.getQueryString();
+            if (q != null && q.length() != 0)
             {
-                int param = buf.lastIndexOf(";");
-                if (param < 0)
-                    buf.append('/');
-                else
-                    buf.insert(param, '/');
-                String q = request.getQueryString();
-                if (q != null && q.length() != 0)
-                {
-                    buf.append('?');
-                    buf.append(q);
-                }
-                response.setContentLength(0);
-                response.sendRedirect(response.encodeRedirectURL(buf.toString()));
+                buf.append('?');
+                buf.append(q);
             }
+            response.setContentLength(0);
+            response.sendRedirect(response.encodeRedirectURL(buf.toString()));
             return;
         }
 
@@ -430,7 +427,7 @@ public class ResourceService
                 return;
             }
 
-            RequestDispatcher dispatcher = context.getRequestDispatcher(welcome);
+            RequestDispatcher dispatcher = context.getRequestDispatcher(URIUtil.encodePath(welcome));
             if (dispatcher != null)
             {
                 // Forward to the index
@@ -722,6 +719,12 @@ public class ResourceService
                             content.release();
                         }
 
+                        @Override
+                        public InvocationType getInvocationType()
+                        {
+                            return InvocationType.NON_BLOCKING;
+                        }
+
                         @Override
                         public String toString()
                         {
