@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2020 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -21,49 +21,48 @@ package org.eclipse.jetty.server;
 import java.io.IOException;
 
 import org.eclipse.jetty.util.annotation.ManagedObject;
-import org.eclipse.jetty.util.log.Slf4jLog;
 
 /**
  * Implementation of NCSARequestLog where output is sent as a SLF4J INFO Log message on the named logger "org.eclipse.jetty.server.RequestLog"
+ *
+ * @deprecated use {@link CustomRequestLog} given format string {@link CustomRequestLog#EXTENDED_NCSA_FORMAT} with an {@link Slf4jRequestLogWriter}
  */
+@Deprecated
 @ManagedObject("NCSA standard format request log to slf4j bridge")
-public class Slf4jRequestLog extends AbstractNCSARequestLog implements RequestLog
+public class Slf4jRequestLog extends AbstractNCSARequestLog
 {
-    private Slf4jLog logger;
-    private String loggerName;
+    private final Slf4jRequestLogWriter _requestLogWriter;
 
     public Slf4jRequestLog()
     {
-        // Default logger name (can be set)
-        this.loggerName = "org.eclipse.jetty.server.RequestLog";
+        this(new Slf4jRequestLogWriter());
+    }
+
+    public Slf4jRequestLog(Slf4jRequestLogWriter writer)
+    {
+        super(writer);
+        _requestLogWriter = writer;
     }
 
     public void setLoggerName(String loggerName)
     {
-        this.loggerName = loggerName;
+        _requestLogWriter.setLoggerName(loggerName);
     }
 
     public String getLoggerName()
     {
-        return loggerName;
+        return _requestLogWriter.getLoggerName();
     }
 
     @Override
     protected boolean isEnabled()
     {
-        return logger != null;
+        return _requestLogWriter.isEnabled();
     }
 
     @Override
     public void write(String requestEntry) throws IOException
     {
-        logger.info(requestEntry);
-    }
-
-    @Override
-    protected synchronized void doStart() throws Exception
-    {
-        logger = new Slf4jLog(loggerName);
-        super.doStart();
+        _requestLogWriter.write(requestEntry);
     }
 }
