@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2019 Mort Bay Consulting Pty. Ltd.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -24,7 +24,7 @@ import org.eclipse.jetty.server.Request;
 
 public interface GzipFactory
 {
-    Deflater getDeflater(Request request, long content_length);
+    Deflater getDeflater(Request request, long contentLength);
 
     boolean isMimeTypeGzipable(String mimetype);
 
