@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2020 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -21,6 +21,7 @@ package org.eclipse.jetty.server.session;
 import java.util.Collections;
 import java.util.HashSet;
 import java.util.Set;
+import java.util.concurrent.atomic.AtomicReference;
 import java.util.function.Function;
 import javax.servlet.http.HttpServletRequest;
 
@@ -98,6 +99,12 @@ public abstract class AbstractSessionCache extends ContainerLifeCycle implements
      * a dirty session will be flushed to the session store.
      */
     protected boolean _flushOnResponseCommit;
+    
+    /**
+     * If true, when the server shuts down, all sessions in the
+     * cache will be invalidated before being removed.
+     */
+    protected boolean _invalidateOnShutdown;
 
     /**
      * Create a new Session object from pre-existing session data
@@ -334,12 +341,13 @@ public abstract class AbstractSessionCache extends ContainerLifeCycle implements
      * 
      * @param id The session to retrieve
      * @param enter if true, the usage count of the session will be incremented
-     * @return
-     * @throws Exception
+     * @return the session if it exists, null otherwise
+     * @throws Exception if the session cannot be loaded
      */
     protected Session getAndEnter(String id, boolean enter) throws Exception
     {
         Session session = null;
+        AtomicReference<Exception> exception = new AtomicReference<Exception>();
 
         session = doComputeIfAbsent(id, k ->
         {
@@ -365,11 +373,15 @@ public abstract class AbstractSessionCache extends ContainerLifeCycle implements
             }
             catch (Exception e)
             {
-                LOG.warn("Error loading session {}", id, e);
+                exception.set(e);
                 return null;
             }
         });
 
+        Exception ex = exception.get();
+        if (ex != null)
+            throw ex;
+            
         if (session != null)
         {
             try (Lock lock = session.lock())
@@ -412,7 +424,7 @@ public abstract class AbstractSessionCache extends ContainerLifeCycle implements
             if (data == null) //session doesn't exist
                 return null;
 
-            data.setLastNode(_context.getWorkerName());//we are going to manage the node
+            data.setLastNode(_context.getWorkerName()); //we are going to manage the node
             session = newSession(data);
             return session;
         }
@@ -473,7 +485,7 @@ public abstract class AbstractSessionCache extends ContainerLifeCycle implements
         {
             //only write the session out at this point if the attributes changed. If only
             //the lastAccess/expiry time changed defer the write until the last request exits
-            if (session.getSessionData().isDirty() && _flushOnResponseCommit)
+            if (session.isValid() && session.getSessionData().isDirty() && _flushOnResponseCommit)
             {
                 if (LOG.isDebugEnabled())
                     LOG.debug("Flush session {} on response commit", session);
@@ -578,7 +590,7 @@ public abstract class AbstractSessionCache extends ContainerLifeCycle implements
                         //reactivate the session
                         session.didActivate();
                         session.setResident(true);
-                        doPutIfAbsent(id, session);//ensure it is in our map
+                        doPutIfAbsent(id, session); //ensure it is in our map
                         if (LOG.isDebugEnabled())
                             LOG.debug("Session reactivated id={}", id);
                     }
@@ -741,7 +753,8 @@ public abstract class AbstractSessionCache extends ContainerLifeCycle implements
                 }
                 catch (Exception e)
                 {
-                    LOG.warn("Passivation of idle session {} failed", session.getId(), e);
+                    LOG.warn("Passivation of idle session {} failed", session.getId());
+                    LOG.warn(e);
                 }
             }
         }
@@ -808,6 +821,18 @@ public abstract class AbstractSessionCache extends ContainerLifeCycle implements
         _saveOnInactiveEviction = saveOnEvict;
     }
 
+    @Override
+    public void setInvalidateOnShutdown(boolean invalidateOnShutdown)
+    {
+        _invalidateOnShutdown = invalidateOnShutdown;
+    }
+
+    @Override
+    public boolean isInvalidateOnShutdown()
+    {
+        return _invalidateOnShutdown;
+    }
+
     /**
      * Whether we should save a session that has been inactive before
      * we boot it from the cache.
@@ -838,7 +863,8 @@ public abstract class AbstractSessionCache extends ContainerLifeCycle implements
         }
         catch (Exception e)
         {
-            LOG.warn("Save of new session {} failed", id, e);
+            LOG.warn("Save of new session {} failed", id);
+            LOG.warn(e);
         }
         return session;
     }
