@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2021 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -110,6 +110,8 @@ public class DumpHandler extends AbstractHandler
         writer.write("<pre>\ncontentType=" + request.getContentType() + "\n</pre>\n");
         writer.write("<pre>\nencoding=" + request.getCharacterEncoding() + "\n</pre>\n");
         writer.write("<pre>\nservername=" + request.getServerName() + "\n</pre>\n");
+        writer.write("<pre>\nserverport=" + request.getServerPort() + "\n</pre>\n");
+        writer.write("<pre>\nlocalname=" + request.getLocalName() + "\n</pre>\n");
         writer.write("<pre>\nlocal=" + request.getLocalAddr() + ":" + request.getLocalPort() + "\n</pre>\n");
         writer.write("<pre>\nremote=" + request.getRemoteAddr() + ":" + request.getRemotePort() + "\n</pre>\n");
         writer.write("<h3>Header:</h3><pre>");
