@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2021 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -26,6 +26,9 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.Reader;
 import java.io.UnsupportedEncodingException;
+import java.net.Inet6Address;
+import java.net.InetAddress;
+import java.net.InetSocketAddress;
 import java.nio.charset.StandardCharsets;
 import java.nio.file.Files;
 import java.nio.file.Path;
@@ -52,6 +55,8 @@ import org.eclipse.jetty.http.HttpCompliance;
 import org.eclipse.jetty.http.HttpComplianceSection;
 import org.eclipse.jetty.http.HttpTester;
 import org.eclipse.jetty.http.MimeTypes;
+import org.eclipse.jetty.io.Connection;
+import org.eclipse.jetty.io.EndPoint;
 import org.eclipse.jetty.server.LocalConnector.LocalEndPoint;
 import org.eclipse.jetty.server.handler.AbstractHandler;
 import org.eclipse.jetty.server.handler.ContextHandler;
@@ -96,12 +101,37 @@ public class RequestTest
     private Server _server;
     private LocalConnector _connector;
     private RequestHandler _handler;
+    private boolean _normalizeAddress = true;
 
     @BeforeEach
     public void init() throws Exception
     {
         _server = new Server();
-        HttpConnectionFactory http = new HttpConnectionFactory();
+        HttpConnectionFactory http = new HttpConnectionFactory()
+        {
+            @Override
+            public Connection newConnection(Connector connector, EndPoint endPoint)
+            {
+                HttpConnection conn = new HttpConnection(getHttpConfiguration(), connector, endPoint, getHttpCompliance(), isRecordHttpComplianceViolations())
+                {
+                    @Override
+                    protected HttpChannelOverHttp newHttpChannel()
+                    {
+                        return new HttpChannelOverHttp(this, getConnector(), getHttpConfiguration(), getEndPoint(), this)
+                        {
+                            @Override
+                            protected String formatAddrOrHost(String addr)
+                            {
+                                if (_normalizeAddress)
+                                    return super.formatAddrOrHost(addr);
+                                return addr;
+                            }
+                        };
+                    }
+                };
+                return configure(conn, connector, endPoint);
+            }
+        };
         http.setInputBufferSize(1024);
         http.getHttpConfiguration().setRequestHeaderSize(512);
         http.getHttpConfiguration().setResponseHeaderSize(512);
@@ -933,6 +963,65 @@ public class RequestTest
         assertEquals("8888", results.get(i++));
     }
 
+    @Test
+    public void testIPv6() throws Exception
+    {
+        final ArrayList<String> results = new ArrayList<>();
+        final InetAddress local = Inet6Address.getByAddress("localIPv6", new byte[]{
+            0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 0, 8
+        });
+        final InetSocketAddress localAddr = new InetSocketAddress(local, 32768);
+        _handler._checker = new RequestTester()
+        {
+            @Override
+            public boolean check(HttpServletRequest request, HttpServletResponse response)
+            {
+                ((Request)request).setRemoteAddr(localAddr);
+                results.add(request.getRemoteAddr());
+                results.add(request.getRemoteHost());
+                results.add(Integer.toString(request.getRemotePort()));
+                results.add(request.getServerName());
+                results.add(Integer.toString(request.getServerPort()));
+                results.add(request.getLocalAddr());
+                results.add(Integer.toString(request.getLocalPort()));
+                return true;
+            }
+        };
+
+        _normalizeAddress = true;
+        String response = _connector.getResponse(
+            "GET / HTTP/1.1\n" +
+                "Host: [::1]:8888\n" +
+                "Connection: close\n" +
+                "\n");
+        int i = 0;
+        assertThat(response, containsString("200 OK"));
+        assertEquals("[1:2:3:4:5:6:7:8]", results.get(i++));
+        assertEquals("localIPv6", results.get(i++));
+        assertEquals("32768", results.get(i++));
+        assertEquals("[::1]", results.get(i++));
+        assertEquals("8888", results.get(i++));
+        assertEquals("0.0.0.0", results.get(i++));
+        assertEquals("0", results.get(i));
+
+        _normalizeAddress = false;
+        results.clear();
+        response = _connector.getResponse(
+            "GET / HTTP/1.1\n" +
+                "Host: [::1]:8888\n" +
+                "Connection: close\n" +
+                "\n");
+        i = 0;
+        assertThat(response, containsString("200 OK"));
+        assertEquals("1:2:3:4:5:6:7:8", results.get(i++));
+        assertEquals("localIPv6", results.get(i++));
+        assertEquals("32768", results.get(i++));
+        assertEquals("[::1]", results.get(i++));
+        assertEquals("8888", results.get(i++));
+        assertEquals("0.0.0.0", results.get(i++));
+        assertEquals("0", results.get(i));
+    }
+
     @Test
     public void testContent() throws Exception
     {
@@ -1616,78 +1705,6 @@ public class RequestTest
         assertEquals("value", cookies.get(0).getValue());
     }
 
-    @Disabled("No longer relevant")
-    @Test
-    public void testCookieLeak() throws Exception
-    {
-        final String[] cookie = new String[10];
-
-        _handler._checker = new RequestTester()
-        {
-            @Override
-            public boolean check(HttpServletRequest request, HttpServletResponse response)
-            {
-                for (int i = 0; i < cookie.length; i++)
-                {
-                    cookie[i] = null;
-                }
-
-                Cookie[] cookies = request.getCookies();
-                for (int i = 0; cookies != null && i < cookies.length; i++)
-                {
-                    cookie[i] = cookies[i].getValue();
-                }
-                return true;
-            }
-        };
-
-        String request = "POST / HTTP/1.1\r\n" +
-            "Host: whatever\r\n" +
-            "Cookie: other=cookie\r\n" +
-            "\r\n" +
-            "POST / HTTP/1.1\r\n" +
-            "Host: whatever\r\n" +
-            "Cookie: name=value\r\n" +
-            "Connection: close\r\n" +
-            "\r\n";
-
-        _connector.getResponse(request);
-
-        assertEquals("value", cookie[0]);
-        assertEquals(null, cookie[1]);
-
-        request = "POST / HTTP/1.1\r\n" +
-            "Host: whatever\r\n" +
-            "Cookie: name=value\r\n" +
-            "\r\n" +
-            "POST / HTTP/1.1\r\n" +
-            "Host: whatever\r\n" +
-            "Cookie: \r\n" +
-            "Connection: close\r\n" +
-            "\r\n";
-
-        _connector.getResponse(request);
-        assertEquals(null, cookie[0]);
-        assertEquals(null, cookie[1]);
-
-        request = "POST / HTTP/1.1\r\n" +
-            "Host: whatever\r\n" +
-            "Cookie: name=value\r\n" +
-            "Cookie: other=cookie\r\n" +
-            "\r\n" +
-            "POST / HTTP/1.1\r\n" +
-            "Host: whatever\r\n" +
-            "Cookie: name=value\r\n" +
-            "Cookie:\r\n" +
-            "Connection: close\r\n" +
-            "\r\n";
-
-        _connector.getResponse(request);
-
-        assertEquals("value", cookie[0]);
-        assertEquals(null, cookie[1]);
-    }
-
     @Test
     public void testHashDOSKeys() throws Exception
     {
@@ -1837,6 +1854,24 @@ public class RequestTest
         assertEquals(0, request.getParameterMap().size());
     }
 
+    @Test
+    public void testEncoding() throws Exception
+    {
+        _handler._checker = (request, response) -> "/foo/bar".equals(request.getPathInfo());
+        String request = "GET /f%6f%6F/b%u0061r HTTP/1.0\r\n" +
+            "Host: whatever\r\n" +
+            "\r\n";
+
+        _connector.getBean(HttpConnectionFactory.class).setHttpCompliance(HttpCompliance.RFC7230);
+        assertThat(_connector.getResponse(request), startsWith("HTTP/1.1 200"));
+
+        HttpCompliance.CUSTOM0.sections().clear();
+        HttpCompliance.CUSTOM0.sections().addAll(HttpCompliance.RFC7230.sections());
+        HttpCompliance.CUSTOM0.sections().add(HttpComplianceSection.NO_UTF16_ENCODINGS);
+        _connector.getBean(HttpConnectionFactory.class).setHttpCompliance(HttpCompliance.CUSTOM0);
+        assertThat(_connector.getResponse(request), startsWith("HTTP/1.1 400"));
+    }
+
     @Test
     public void testAmbiguousParameters() throws Exception
     {
@@ -1895,6 +1930,71 @@ public class RequestTest
         assertThat(_connector.getResponse(request), startsWith("HTTP/1.1 200"));
     }
 
+    public void setComplianceModes(HttpComplianceSection... complianceSections)
+    {
+        setComplianceModes(null, complianceSections);
+    }
+
+    public void setComplianceModes(HttpCompliance compliance, HttpComplianceSection... additionalSections)
+    {
+        HttpCompliance.CUSTOM0.sections().clear();
+        if (compliance != null)
+            HttpCompliance.CUSTOM0.sections().addAll(compliance.sections());
+        HttpCompliance.CUSTOM0.sections().addAll(Arrays.asList(additionalSections));
+        _connector.getBean(HttpConnectionFactory.class).setHttpCompliance(HttpCompliance.CUSTOM0);
+    }
+
+    @Test
+    public void testAmbiguousPaths() throws Exception
+    {
+        // TODO this is a poor test as it is not very exhaustive.  See HttpURITest for better tests.
+        _handler._checker = (request, response) ->
+        {
+            response.getOutputStream().println("servletPath=" + request.getServletPath());
+            response.getOutputStream().println("pathInfo=" + request.getPathInfo());
+            return true;
+        };
+        String request = "GET /unnormal/.././path/ambiguous%2f%2e%2e/info HTTP/1.0\r\n" +
+            "Host: whatever\r\n" +
+            "\r\n";
+
+        setComplianceModes(HttpCompliance.RFC7230);
+        assertThat(_connector.getResponse(request), Matchers.allOf(
+            startsWith("HTTP/1.1 200"),
+            containsString("pathInfo=/path/info")));
+
+        setComplianceModes(HttpComplianceSection.NO_AMBIGUOUS_PATH_SEPARATORS);
+        assertThat(_connector.getResponse(request), startsWith("HTTP/1.1 400"));
+    }
+
+    @Test
+    public void testAmbiguousEncoding() throws Exception
+    {
+        _handler._checker = (request, response) -> true;
+        String request = "GET /ambiguous/encoded/%25/path HTTP/1.0\r\n" +
+            "Host: whatever\r\n" +
+            "\r\n";
+
+        setComplianceModes(HttpCompliance.RFC7230);
+        assertThat(_connector.getResponse(request), startsWith("HTTP/1.1 200"));
+        setComplianceModes(HttpCompliance.RFC7230, HttpComplianceSection.NO_AMBIGUOUS_PATH_ENCODING);
+        assertThat(_connector.getResponse(request), startsWith("HTTP/1.1 400"));
+    }
+
+    @Test
+    public void testAmbiguousDoubleSlash() throws Exception
+    {
+        _handler._checker = (request, response) -> true;
+        String request = "GET /ambiguous/doubleSlash// HTTP/1.0\r\n" +
+            "Host: whatever\r\n" +
+            "\r\n";
+
+        setComplianceModes(HttpCompliance.RFC7230);
+        assertThat(_connector.getResponse(request), startsWith("HTTP/1.1 200"));
+        setComplianceModes(HttpCompliance.RFC7230, HttpComplianceSection.NO_AMBIGUOUS_EMPTY_SEGMENT);
+        assertThat(_connector.getResponse(request), startsWith("HTTP/1.1 400"));
+    }
+
     private static long getFileCount(Path path)
     {
         try (Stream<Path> s = Files.list(path))
