@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2021 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -134,11 +134,14 @@ public class DirConfigSource implements ConfigSource
 
                 List<Path> paths = new ArrayList<>();
 
-                for (Path diniFile : Files.newDirectoryStream(startDdir, filter))
+                try (DirectoryStream<Path> dirStream = Files.newDirectoryStream(startDdir, filter))
                 {
-                    if (FS.canReadFile(diniFile))
+                    for (Path diniFile : dirStream)
                     {
-                        paths.add(diniFile);
+                        if (FS.canReadFile(diniFile))
+                        {
+                            paths.add(diniFile);
+                        }
                     }
                 }
 
