@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2019 Mort Bay Consulting Pty. Ltd.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -43,9 +43,8 @@ import java.util.concurrent.locks.ReentrantLock;
  * The queue has both a capacity (the size of the array currently allocated) and a max capacity (the maximum size that may be allocated), which defaults to
  * {@link Integer#MAX_VALUE}.
  * </p>
- * 
- * @param <E>
- *            The element type
+ *
+ * @param <E> The element type
  */
 public class BlockingArrayQueue<E> extends AbstractList<E> implements BlockingQueue<E>
 {
@@ -81,7 +80,7 @@ public class BlockingArrayQueue<E> extends AbstractList<E> implements BlockingQu
 
     /**
      * Creates an unbounded {@link BlockingArrayQueue} with default initial capacity and grow factor.
-     * 
+     *
      * @see #DEFAULT_CAPACITY
      * @see #DEFAULT_GROWTH
      */
@@ -94,9 +93,8 @@ public class BlockingArrayQueue<E> extends AbstractList<E> implements BlockingQu
 
     /**
      * Creates a bounded {@link BlockingArrayQueue} that does not grow. The capacity of the queue is fixed and equal to the given parameter.
-     * 
-     * @param maxCapacity
-     *            the maximum capacity
+     *
+     * @param maxCapacity the maximum capacity
      */
     public BlockingArrayQueue(int maxCapacity)
     {
@@ -107,11 +105,9 @@ public class BlockingArrayQueue<E> extends AbstractList<E> implements BlockingQu
 
     /**
      * Creates an unbounded {@link BlockingArrayQueue} that grows by the given parameter.
-     * 
-     * @param capacity
-     *            the initial capacity
-     * @param growBy
-     *            the growth factor
+     *
+     * @param capacity the initial capacity
+     * @param growBy the growth factor
      */
     public BlockingArrayQueue(int capacity, int growBy)
     {
@@ -122,13 +118,10 @@ public class BlockingArrayQueue<E> extends AbstractList<E> implements BlockingQu
 
     /**
      * Create a bounded {@link BlockingArrayQueue} that grows by the given parameter.
-     * 
-     * @param capacity
-     *            the initial capacity
-     * @param growBy
-     *            the growth factor
-     * @param maxCapacity
-     *            the maximum capacity
+     *
+     * @param capacity the initial capacity
+     * @param growBy the growth factor
+     * @param maxCapacity the maximum capacity
      */
     public BlockingArrayQueue(int capacity, int growBy, int maxCapacity)
     {
@@ -139,9 +132,8 @@ public class BlockingArrayQueue<E> extends AbstractList<E> implements BlockingQu
         _maxCapacity = maxCapacity;
     }
 
-    /*----------------------------------------------------------------------------*/
+
     /* Collection methods */
-    /*----------------------------------------------------------------------------*/
 
     @Override
     public void clear()
@@ -181,9 +173,8 @@ public class BlockingArrayQueue<E> extends AbstractList<E> implements BlockingQu
         return listIterator();
     }
 
-    /*----------------------------------------------------------------------------*/
+
     /* Queue methods */
-    /*----------------------------------------------------------------------------*/
 
     @SuppressWarnings("unchecked")
     @Override
@@ -254,9 +245,8 @@ public class BlockingArrayQueue<E> extends AbstractList<E> implements BlockingQu
         return e;
     }
 
-    /*----------------------------------------------------------------------------*/
+
     /* BlockingQueue methods */
-    /*----------------------------------------------------------------------------*/
 
     @Override
     public boolean offer(E e)
@@ -352,10 +342,10 @@ public class BlockingArrayQueue<E> extends AbstractList<E> implements BlockingQu
                     _notEmpty.await();
                 }
             }
-            catch (InterruptedException ie)
+            catch (InterruptedException ex)
             {
                 _notEmpty.signal();
-                throw ie;
+                throw ex;
             }
 
             final int head = _indexes[HEAD_OFFSET];
@@ -434,9 +424,9 @@ public class BlockingArrayQueue<E> extends AbstractList<E> implements BlockingQu
                 int i = head;
                 while (true)
                 {
-                    if (Objects.equals(_elements[i],o))
+                    if (Objects.equals(_elements[i], o))
                     {
-                        remove(i >= head?i - head:capacity - head + i);
+                        remove(i >= head ? i - head : capacity - head + i);
                         return true;
                     }
                     ++i;
@@ -484,18 +474,59 @@ public class BlockingArrayQueue<E> extends AbstractList<E> implements BlockingQu
     @Override
     public int drainTo(Collection<? super E> c)
     {
-        throw new UnsupportedOperationException();
+        return drainTo(c, Integer.MAX_VALUE);
     }
 
     @Override
     public int drainTo(Collection<? super E> c, int maxElements)
     {
-        throw new UnsupportedOperationException();
+        int elements = 0;
+        _tailLock.lock();
+        try
+        {
+            _headLock.lock();
+            try
+            {
+                final int head = _indexes[HEAD_OFFSET];
+                final int tail = _indexes[TAIL_OFFSET];
+                final int capacity = _elements.length;
+
+                int i = head;
+                while (i != tail && elements < maxElements)
+                {
+                    elements++;
+                    c.add((E)_elements[i]);
+                    ++i;
+                    if (i == capacity)
+                        i = 0;
+                }
+
+                if (i == tail)
+                {
+                    _indexes[HEAD_OFFSET] = 0;
+                    _indexes[TAIL_OFFSET] = 0;
+                    _size.set(0);
+                }
+                else
+                {
+                    _indexes[HEAD_OFFSET] = i;
+                    _size.addAndGet(-elements);
+                }
+            }
+            finally
+            {
+                _headLock.unlock();
+            }
+        }
+        finally
+        {
+            _tailLock.unlock();
+        }
+        return elements;
     }
 
-    /*----------------------------------------------------------------------------*/
+
     /* List methods */
-    /*----------------------------------------------------------------------------*/
 
     @SuppressWarnings("unchecked")
     @Override
@@ -505,7 +536,6 @@ public class BlockingArrayQueue<E> extends AbstractList<E> implements BlockingQu
         _tailLock.lock();
         try
         {
-
             _headLock.lock();
             try
             {
@@ -569,18 +599,18 @@ public class BlockingArrayQueue<E> extends AbstractList<E> implements BlockingQu
 
                     if (i < tail)
                     {
-                        System.arraycopy(_elements,i,_elements,i + 1,tail - i);
+                        System.arraycopy(_elements, i, _elements, i + 1, tail - i);
                         _elements[i] = e;
                     }
                     else
                     {
                         if (tail > 0)
                         {
-                            System.arraycopy(_elements,0,_elements,1,tail);
+                            System.arraycopy(_elements, 0, _elements, 1, tail);
                             _elements[0] = _elements[capacity - 1];
                         }
 
-                        System.arraycopy(_elements,i,_elements,i + 1,capacity - i - 1);
+                        System.arraycopy(_elements, i, _elements, i + 1, capacity - i - 1);
                         _elements[i] = e;
                     }
                 }
@@ -655,16 +685,16 @@ public class BlockingArrayQueue<E> extends AbstractList<E> implements BlockingQu
                 int tail = _indexes[TAIL_OFFSET];
                 if (i < tail)
                 {
-                    System.arraycopy(_elements,i + 1,_elements,i,tail - i);
+                    System.arraycopy(_elements, i + 1, _elements, i, tail - i);
                     --_indexes[TAIL_OFFSET];
                 }
                 else
                 {
-                    System.arraycopy(_elements,i + 1,_elements,i,capacity - i - 1);
+                    System.arraycopy(_elements, i + 1, _elements, i, capacity - i - 1);
                     _elements[capacity - 1] = _elements[0];
                     if (tail > 0)
                     {
-                        System.arraycopy(_elements,1,_elements,0,tail);
+                        System.arraycopy(_elements, 1, _elements, 0, tail);
                         --_indexes[TAIL_OFFSET];
                     }
                     else
@@ -707,16 +737,16 @@ public class BlockingArrayQueue<E> extends AbstractList<E> implements BlockingQu
                     int tail = _indexes[TAIL_OFFSET];
                     if (head < tail)
                     {
-                        System.arraycopy(_elements,head,elements,0,tail - head);
+                        System.arraycopy(_elements, head, elements, 0, tail - head);
                     }
                     else
                     {
                         int chunk = _elements.length - head;
-                        System.arraycopy(_elements,head,elements,0,chunk);
-                        System.arraycopy(_elements,0,elements,chunk,tail);
+                        System.arraycopy(_elements, head, elements, 0, chunk);
+                        System.arraycopy(_elements, 0, elements, chunk, tail);
                     }
                 }
-                return new Itr(elements,index);
+                return new Itr(elements, index);
             }
             finally
             {
@@ -729,10 +759,6 @@ public class BlockingArrayQueue<E> extends AbstractList<E> implements BlockingQu
         }
     }
 
-    /*----------------------------------------------------------------------------*/
-    /* Additional methods */
-    /*----------------------------------------------------------------------------*/
-
     /**
      * @return the current capacity of this queue
      */
@@ -757,10 +783,6 @@ public class BlockingArrayQueue<E> extends AbstractList<E> implements BlockingQu
         return _maxCapacity;
     }
 
-    /*----------------------------------------------------------------------------*/
-    /* Implementation methods */
-    /*----------------------------------------------------------------------------*/
-
     private boolean grow()
     {
         if (_growCapacity <= 0)
@@ -783,14 +805,14 @@ public class BlockingArrayQueue<E> extends AbstractList<E> implements BlockingQu
                 if (head < tail)
                 {
                     newTail = tail - head;
-                    System.arraycopy(_elements,head,elements,0,newTail);
+                    System.arraycopy(_elements, head, elements, 0, newTail);
                 }
                 else if (head > tail || _size.get() > 0)
                 {
                     newTail = capacity + tail - head;
                     int cut = capacity - head;
-                    System.arraycopy(_elements,head,elements,0,cut);
-                    System.arraycopy(_elements,0,elements,cut,tail);
+                    System.arraycopy(_elements, head, elements, 0, cut);
+                    System.arraycopy(_elements, 0, elements, cut, tail);
                 }
                 else
                 {
