@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2021 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -120,7 +120,7 @@ public class URIUtil
                         buf = new StringBuilder(path.length() * 2);
                         break loop;
                     default:
-                        if (c > 127)
+                        if (c < 0x20 || c >= 0x7f)
                         {
                             bytes = path.getBytes(URIUtil.__CHARSET);
                             buf = new StringBuilder(path.length() * 2);
@@ -193,7 +193,7 @@ public class URIUtil
                     continue;
 
                 default:
-                    if (c > 127)
+                    if (c < 0x20 || c >= 0x7f)
                     {
                         bytes = path.getBytes(URIUtil.__CHARSET);
                         break loop;
@@ -261,7 +261,7 @@ public class URIUtil
                         buf.append("%7D");
                         continue;
                     default:
-                        if (c < 0)
+                        if (c < 0x20 || c >= 0x7f)
                         {
                             buf.append('%');
                             TypeUtil.toHex(c, buf);
@@ -475,8 +475,8 @@ public class URIUtil
                             char u = path.charAt(i + 1);
                             if (u == 'u')
                             {
-                                // TODO remove %u support in jetty-10
-                                // this is wrong. This is a codepoint not a char
+                                // In Jetty-10 UTF16 encoding is only supported with UriCompliance.Violation.UTF16_ENCODINGS.
+                                // This is wrong. This is a codepoint not a char
                                 builder.append((char)(0xffff & TypeUtil.parseInt(path, i + 2, 4, 16)));
                                 i += 5;
                             }
@@ -537,7 +537,6 @@ public class URIUtil
         {
             throw new IllegalArgumentException("cannot decode URI", e);
         }
-
     }
 
     /* Decode a URI path and strip parameters of ISO-8859-1 path
@@ -562,8 +561,7 @@ public class URIUtil
                         char u = path.charAt(i + 1);
                         if (u == 'u')
                         {
-                            // TODO remove %u encoding support in jetty-10
-                            // This is wrong. This is a codepoint not a char
+                            // In Jetty-10 UTF16 encoding is only supported with UriCompliance.Violation.UTF16_ENCODINGS.                            // This is wrong. This is a codepoint not a char
                             builder.append((char)(0xffff & TypeUtil.parseInt(path, i + 2, 4, 16)));
                             i += 5;
                         }
@@ -782,143 +780,138 @@ public class URIUtil
     }
 
     /**
-     * Convert an encoded path to a canonical form.
+     * Convert a partial URI to a canonical form.
      * <p>
-     * All instances of "." and ".." are factored out.
+     * All segments of "." and ".." are factored out.
      * Null is returned if the path tries to .. above its root.
      * </p>
      *
-     * @param path the path to convert, decoded, with path separators '/' and no queries.
+     * @param uri the encoded URI from the path onwards, which may contain query strings and/or fragments
      * @return the canonical path, or null if path traversal above root.
+     * @see #canonicalPath(String)
      */
-    public static String canonicalPath(String path)
+    public static String canonicalURI(String uri)
     {
-        // See https://tools.ietf.org/html/rfc3986#section-5.2.4
+        if (uri == null || uri.isEmpty())
+            return uri;
 
-        if (path == null || path.isEmpty())
-            return path;
-
-        int end = path.length();
+        boolean slash = true;
+        int end = uri.length();
         int i = 0;
-        int dots = 0;
 
+        // Initially just loop looking if we may need to normalize
         loop: while (i < end)
         {
-            char c = path.charAt(i);
+            char c = uri.charAt(i);
             switch (c)
             {
                 case '/':
-                    dots = 0;
+                    slash = true;
                     break;
 
                 case '.':
-                    if (dots == 0)
-                    {
-                        dots = 1;
+                    if (slash)
                         break loop;
-                    }
-                    dots = -1;
+                    slash = false;
                     break;
 
+                case '?':
+                case '#':
+                    // Nothing to normalize so return original path
+                    return uri;
+
                 default:
-                    dots = -1;
+                    slash = false;
             }
 
             i++;
         }
 
+        // Nothing to normalize so return original path
         if (i == end)
-            return path;
+            return uri;
 
-        StringBuilder canonical = new StringBuilder(path.length());
-        canonical.append(path, 0, i);
+        // We probably need to normalize, so copy to path so far into builder
+        StringBuilder canonical = new StringBuilder(uri.length());
+        canonical.append(uri, 0, i);
 
+        // Loop looking for single and double dot segments
+        int dots = 1;
         i++;
-        while (i <= end)
+        loop : while (i < end)
         {
-            char c = i < end ? path.charAt(i) : '\0';
+            char c = uri.charAt(i);
             switch (c)
             {
-                case '\0':
-                    if (dots == 2)
-                    {
-                        if (canonical.length() < 2)
-                            return null;
-                        canonical.setLength(canonical.length() - 1);
-                        canonical.setLength(canonical.lastIndexOf("/") + 1);
-                    }
-                    break;
-
                 case '/':
-                    switch (dots)
-                    {
-                        case 1:
-                            break;
-
-                        case 2:
-                            if (canonical.length() < 2)
-                                return null;
-                            canonical.setLength(canonical.length() - 1);
-                            canonical.setLength(canonical.lastIndexOf("/") + 1);
-                            break;
-
-                        default:
-                            canonical.append(c);
-                    }
+                    if (doDotsSlash(canonical, dots))
+                        return null;
+                    slash = true;
                     dots = 0;
                     break;
 
+                case '?':
+                case '#':
+                    // finish normalization at a query
+                    break loop;
+
                 case '.':
-                    switch (dots)
-                    {
-                        case 0:
-                            dots = 1;
-                            break;
-                        case 1:
-                            dots = 2;
-                            break;
-                        case 2:
-                            canonical.append("...");
-                            dots = -1;
-                            break;
-                        default:
-                            canonical.append('.');
-                    }
+                    // Count dots only if they are leading in the segment
+                    if (dots > 0)
+                        dots++;
+                    else if (slash)
+                        dots = 1;
+                    else
+                        canonical.append('.');
+                    slash = false;
                     break;
 
                 default:
-                    switch (dots)
-                    {
-                        case 1:
-                            canonical.append('.');
-                            break;
-                        case 2:
-                            canonical.append("..");
-                            break;
-                        default:
-                    }
+                    // Add leading dots to the path
+                    while (dots-- > 0)
+                        canonical.append('.');
                     canonical.append(c);
-                    dots = -1;
+                    dots = 0;
+                    slash = false;
             }
-
             i++;
         }
+
+        // process any remaining dots
+        if (doDots(canonical, dots))
+            return null;
+
+        // append any query
+        if (i < end)
+            canonical.append(uri, i, end);
+
         return canonical.toString();
     }
 
     /**
-     * Convert a path to a cananonical form.
-     * <p>
-     * All instances of "." and ".." are factored out.
-     * </p>
+     * @param path the encoded URI from the path onwards, which may contain query strings and/or fragments
+     * @return the canonical path, or null if path traversal above root.
+     * @deprecated Use {@link #canonicalURI(String)}
+     */
+    @Deprecated
+    public static String canonicalEncodedPath(String path)
+    {
+        return canonicalURI(path);
+    }
+
+    /**
+     * Convert a decoded URI path to a canonical form.
      * <p>
+     * All segments of "." and ".." are factored out.
      * Null is returned if the path tries to .. above its root.
      * </p>
      *
-     * @param path the path to convert (expects URI/URL form, encoded, and with path separators '/')
+     * @param path the decoded URI path to convert. Any special characters (e.g. '?', "#") are assumed to be part of
+     * the path segments.
      * @return the canonical path, or null if path traversal above root.
+     * @see #canonicalURI(String)
      */
-    public static String canonicalEncodedPath(String path)
+    public static String canonicalPath(String path)
     {
         if (path == null || path.isEmpty())
             return path;
@@ -927,8 +920,8 @@ public class URIUtil
         int end = path.length();
         int i = 0;
 
-        loop:
-        while (i < end)
+        // Initially just loop looking if we may need to normalize
+        loop: while (i < end)
         {
             char c = path.charAt(i);
             switch (c)
@@ -943,9 +936,6 @@ public class URIUtil
                     slash = false;
                     break;
 
-                case '?':
-                    return path;
-
                 default:
                     slash = false;
             }
@@ -953,56 +943,31 @@ public class URIUtil
             i++;
         }
 
+        // Nothing to normalize so return original path
         if (i == end)
             return path;
 
+        // We probably need to normalize, so copy to path so far into builder
         StringBuilder canonical = new StringBuilder(path.length());
         canonical.append(path, 0, i);
 
+        // Loop looking for single and double dot segments
         int dots = 1;
         i++;
-        while (i <= end)
+        while (i < end)
         {
-            char c = i < end ? path.charAt(i) : '\0';
+            char c = path.charAt(i);
             switch (c)
             {
-                case '\0':
                 case '/':
-                case '?':
-                    switch (dots)
-                    {
-                        case 0:
-                            if (c != '\0')
-                                canonical.append(c);
-                            break;
-
-                        case 1:
-                            if (c == '?')
-                                canonical.append(c);
-                            break;
-
-                        case 2:
-                            if (canonical.length() < 2)
-                                return null;
-                            canonical.setLength(canonical.length() - 1);
-                            canonical.setLength(canonical.lastIndexOf("/") + 1);
-                            if (c == '?')
-                                canonical.append(c);
-                            break;
-                        default:
-                            while (dots-- > 0)
-                            {
-                                canonical.append('.');
-                            }
-                            if (c != '\0')
-                                canonical.append(c);
-                    }
-
+                    if (doDotsSlash(canonical, dots))
+                        return null;
                     slash = true;
                     dots = 0;
                     break;
 
                 case '.':
+                    // Count dots only if they are leading in the segment
                     if (dots > 0)
                         dots++;
                     else if (slash)
@@ -1013,20 +978,66 @@ public class URIUtil
                     break;
 
                 default:
+                    // Add leading dots to the path
                     while (dots-- > 0)
-                    {
                         canonical.append('.');
-                    }
                     canonical.append(c);
                     dots = 0;
                     slash = false;
             }
-
             i++;
         }
+
+        // process any remaining dots
+        if (doDots(canonical, dots))
+            return null;
+
         return canonical.toString();
     }
 
+    private static boolean doDots(StringBuilder canonical, int dots)
+    {
+        switch (dots)
+        {
+            case 0:
+            case 1:
+                break;
+            case 2:
+                if (canonical.length() < 2)
+                    return true;
+                canonical.setLength(canonical.length() - 1);
+                canonical.setLength(canonical.lastIndexOf("/") + 1);
+                break;
+            default:
+                while (dots-- > 0)
+                    canonical.append('.');
+        }
+        return false;
+    }
+
+    private static boolean doDotsSlash(StringBuilder canonical, int dots)
+    {
+        switch (dots)
+        {
+            case 0:
+                canonical.append('/');
+                break;
+            case 1:
+                break;
+            case 2:
+                if (canonical.length() < 2)
+                    return true;
+                canonical.setLength(canonical.length() - 1);
+                canonical.setLength(canonical.lastIndexOf("/") + 1);
+                break;
+            default:
+                while (dots-- > 0)
+                    canonical.append('.');
+                canonical.append('/');
+        }
+        return false;
+    }
+
     /**
      * Convert a path to a compact form.
      * All instances of "//" and "///" etc. are factored out to single "/"
