@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2020 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -18,6 +18,8 @@
 
 package org.eclipse.jetty.util.component;
 
+import org.eclipse.jetty.util.annotation.ManagedObject;
+import org.eclipse.jetty.util.annotation.ManagedOperation;
 
 /**
  * <p>A Destroyable is an object which can be destroyed.</p>
@@ -25,7 +27,9 @@ package org.eclipse.jetty.util.component;
  * resources over multiple start/stop cycles.   A call to destroy will release all
  * resources and will prevent any further start/stop cycles from being successful.</p>
  */
+@ManagedObject
 public interface Destroyable
 {
+    @ManagedOperation(value = "Destroys this component", impact = "ACTION")
     void destroy();
 }
