@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2021 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -19,22 +19,24 @@
 package org.eclipse.jetty.util;
 
 import java.util.ArrayList;
-import java.util.HashSet;
 import java.util.List;
 import java.util.ListIterator;
-import java.util.Random;
-import java.util.concurrent.ConcurrentLinkedQueue;
+import java.util.Set;
+import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.CyclicBarrier;
+import java.util.concurrent.ThreadLocalRandom;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicBoolean;
+import java.util.concurrent.atomic.AtomicInteger;
 
 import org.hamcrest.Matchers;
 import org.junit.jupiter.api.Test;
-import org.junit.jupiter.api.condition.DisabledIfSystemProperty;
 
+import static org.awaitility.Awaitility.await;
 import static org.hamcrest.MatcherAssert.assertThat;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertTrue;
 import static org.junit.jupiter.api.Assertions.fail;
 
@@ -166,12 +168,12 @@ public class BlockingArrayQueueTest
     }
 
     @Test
-    @DisabledIfSystemProperty(named = "env", matches = "ci") // TODO: SLOW, needs review
     public void testTake() throws Exception
     {
         final String[] data = new String[4];
 
         final BlockingArrayQueue<String> queue = new BlockingArrayQueue<>();
+        CyclicBarrier barrier = new CyclicBarrier(2);
 
         Thread thread = new Thread()
         {
@@ -182,7 +184,7 @@ public class BlockingArrayQueueTest
                 {
                     data[0] = queue.take();
                     data[1] = queue.take();
-                    Thread.sleep(1000);
+                    barrier.await(5, TimeUnit.SECONDS); // Wait until the main thread already called offer().
                     data[2] = queue.take();
                     data[3] = queue.poll(100, TimeUnit.MILLISECONDS);
                 }
@@ -196,35 +198,36 @@ public class BlockingArrayQueueTest
 
         thread.start();
 
-        Thread.sleep(1000);
+        // Wait until the spawned thread is blocked in queue.take().
+        await().atMost(5, TimeUnit.SECONDS).until(() -> thread.getState() == Thread.State.WAITING);
 
         queue.offer("zero");
         queue.offer("one");
         queue.offer("two");
+        barrier.await(5, TimeUnit.SECONDS); // Notify the spawned thread that offer() was called.
         thread.join();
 
         assertEquals("zero", data[0]);
         assertEquals("one", data[1]);
         assertEquals("two", data[2]);
-        assertEquals(null, data[3]);
+        assertNull(data[3]);
     }
 
     @Test
-    @DisabledIfSystemProperty(named = "env", matches = "ci") // TODO: SLOW, needs review
     public void testConcurrentAccess() throws Exception
     {
-        final int THREADS = 50;
+        final int THREADS = 32;
         final int LOOPS = 1000;
 
-        final BlockingArrayQueue<Integer> queue = new BlockingArrayQueue<>(1 + THREADS * LOOPS);
+        BlockingArrayQueue<Integer> queue = new BlockingArrayQueue<>(1 + THREADS * LOOPS);
 
-        final ConcurrentLinkedQueue<Integer> produced = new ConcurrentLinkedQueue<>();
-        final ConcurrentLinkedQueue<Integer> consumed = new ConcurrentLinkedQueue<>();
+        Set<Integer> produced = ConcurrentHashMap.newKeySet();
+        Set<Integer> consumed = ConcurrentHashMap.newKeySet();
 
-        final AtomicBoolean running = new AtomicBoolean(true);
+        AtomicBoolean consumersRunning = new AtomicBoolean(true);
 
         // start consumers
-        final CyclicBarrier barrier0 = new CyclicBarrier(THREADS + 1);
+        CyclicBarrier consumersBarrier = new CyclicBarrier(THREADS + 1);
         for (int i = 0; i < THREADS; i++)
         {
             new Thread()
@@ -232,20 +235,18 @@ public class BlockingArrayQueueTest
                 @Override
                 public void run()
                 {
-                    final Random random = new Random();
-
                     setPriority(getPriority() - 1);
                     try
                     {
-                        while (running.get())
+                        while (consumersRunning.get())
                         {
-                            int r = 1 + random.nextInt(10);
+                            int r = 1 + ThreadLocalRandom.current().nextInt(10);
                             if (r % 2 == 0)
                             {
                                 Integer msg = queue.poll();
                                 if (msg == null)
                                 {
-                                    Thread.sleep(1 + random.nextInt(10));
+                                    Thread.sleep(ThreadLocalRandom.current().nextInt(2));
                                     continue;
                                 }
                                 consumed.add(msg);
@@ -266,7 +267,7 @@ public class BlockingArrayQueueTest
                     {
                         try
                         {
-                            barrier0.await();
+                            consumersBarrier.await();
                         }
                         catch (Exception e)
                         {
@@ -278,7 +279,7 @@ public class BlockingArrayQueueTest
         }
 
         // start producers
-        final CyclicBarrier barrier1 = new CyclicBarrier(THREADS + 1);
+        CyclicBarrier producersBarrier = new CyclicBarrier(THREADS + 1);
         for (int i = 0; i < THREADS; i++)
         {
             final int id = i;
@@ -287,16 +288,15 @@ public class BlockingArrayQueueTest
                 @Override
                 public void run()
                 {
-                    final Random random = new Random();
                     try
                     {
                         for (int j = 0; j < LOOPS; j++)
                         {
-                            Integer msg = random.nextInt();
+                            Integer msg = ThreadLocalRandom.current().nextInt();
                             produced.add(msg);
                             if (!queue.offer(msg))
                                 throw new Exception(id + " FULL! " + queue.size());
-                            Thread.sleep(1 + random.nextInt(10));
+                            Thread.sleep(ThreadLocalRandom.current().nextInt(2));
                         }
                     }
                     catch (Exception e)
@@ -307,7 +307,7 @@ public class BlockingArrayQueueTest
                     {
                         try
                         {
-                            barrier1.await();
+                            producersBarrier.await();
                         }
                         catch (Exception e)
                         {
@@ -318,22 +318,22 @@ public class BlockingArrayQueueTest
             }.start();
         }
 
-        barrier1.await();
-        int size = queue.size();
-        int last = size - 1;
-        while (size > 0 && size != last)
+        producersBarrier.await();
+
+        AtomicInteger last = new AtomicInteger(queue.size() - 1);
+        await().atMost(5, TimeUnit.SECONDS).until(() ->
         {
-            last = size;
-            Thread.sleep(500);
-            size = queue.size();
-        }
-        running.set(false);
-        barrier0.await();
+            int size = queue.size();
+            if (size == 0 && last.get() == size)
+                return true;
+            last.set(size);
+            return false;
+        });
 
-        HashSet<Integer> prodSet = new HashSet<>(produced);
-        HashSet<Integer> consSet = new HashSet<>(consumed);
+        consumersRunning.set(false);
+        consumersBarrier.await();
 
-        assertEquals(prodSet, consSet);
+        assertEquals(produced, consumed);
     }
 
     @Test
