@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2021 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -22,6 +22,7 @@ import java.io.File;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.OutputStream;
+import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.PathMatcher;
 import java.util.List;
@@ -35,66 +36,56 @@ import org.eclipse.jetty.util.Scanner.Notification;
 import org.hamcrest.Matchers;
 import org.junit.jupiter.api.AfterAll;
 import org.junit.jupiter.api.BeforeAll;
+import org.junit.jupiter.api.Tag;
 import org.junit.jupiter.api.Test;
-import org.junit.jupiter.api.condition.DisabledIfSystemProperty;
-import org.junit.jupiter.api.condition.DisabledOnOs;
 
 import static org.hamcrest.MatcherAssert.assertThat;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
+import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertTrue;
-import static org.junit.jupiter.api.condition.OS.WINDOWS;
 
 public class ScannerTest
 {
-    static File _directory;
+    static Path _directory;
     static Scanner _scanner;
-    static BlockingQueue<Event> _queue = new LinkedBlockingQueue<Event>();
-    static BlockingQueue<List<String>> _bulk = new LinkedBlockingQueue<List<String>>();
+    static BlockingQueue<Event> _queue = new LinkedBlockingQueue<>();
+    static BlockingQueue<List<String>> _bulk = new LinkedBlockingQueue<>();
 
     @BeforeAll
     public static void setUpBeforeClass() throws Exception
     {
-        File testDir = MavenTestingUtils.getTargetTestingDir(ScannerTest.class.getSimpleName());
-        FS.ensureEmpty(testDir);
+        _directory = MavenTestingUtils.getTargetTestingPath(ScannerTest.class.getSimpleName());
+        FS.ensureEmpty(_directory);
 
-        // Use full path, pointing to a real directory (for FileSystems that are case-insensitive, like Windows and OSX to use)
-        // This is only needed for the various comparisons below to make sense.
-        _directory = testDir.toPath().toRealPath().toFile();
+        _directory = _directory.toRealPath();
 
         _scanner = new Scanner();
-        _scanner.addScanDir(_directory);
+        _scanner.addDirectory(_directory);
         _scanner.setScanInterval(0);
         _scanner.setReportDirs(false);
         _scanner.setReportExistingFilesOnStartup(false);
         _scanner.addListener(new Scanner.DiscreteListener()
         {
             @Override
-            public void fileRemoved(String filename) throws Exception
+            public void fileRemoved(String filename)
             {
                 _queue.add(new Event(filename, Notification.REMOVED));
             }
 
             @Override
-            public void fileChanged(String filename) throws Exception
+            public void fileChanged(String filename)
             {
                 _queue.add(new Event(filename, Notification.CHANGED));
             }
 
             @Override
-            public void fileAdded(String filename) throws Exception
+            public void fileAdded(String filename)
             {
                 _queue.add(new Event(filename, Notification.ADDED));
             }
         });
-        _scanner.addListener(new Scanner.BulkListener()
-        {
-            @Override
-            public void filesChanged(List<String> filenames) throws Exception
-            {
-                _bulk.add(filenames);
-            }
-        });
+        _scanner.addListener((Scanner.BulkListener)filenames -> _bulk.add(filenames));
 
         _scanner.start();
         _scanner.scan();
@@ -107,7 +98,6 @@ public class ScannerTest
     public static void tearDownAfterClass() throws Exception
     {
         _scanner.stop();
-        IO.delete(_directory);
     }
 
     static class Event
@@ -125,7 +115,7 @@ public class ScannerTest
     @Test
     public void testDepth() throws Exception
     {
-        File root = new File(_directory, "root");
+        File root = new File(_directory.toFile(), "root");
         FS.ensureDirExists(root);
         FS.touch(new File(root, "foo.foo"));
         FS.touch(new File(root, "foo2.foo"));
@@ -142,7 +132,7 @@ public class ScannerTest
         File y2 = new File(dir2, "yyy2.foo");
         FS.touch(y2);
 
-        BlockingQueue<Event> queue = new LinkedBlockingQueue<Event>();
+        BlockingQueue<Event> queue = new LinkedBlockingQueue<>();
         Scanner scanner = new Scanner();
         scanner.setScanInterval(0);
         scanner.setScanDepth(0);
@@ -152,19 +142,19 @@ public class ScannerTest
         scanner.addListener(new Scanner.DiscreteListener()
         {
             @Override
-            public void fileRemoved(String filename) throws Exception
+            public void fileRemoved(String filename)
             {
                 queue.add(new Event(filename, Notification.REMOVED));
             }
 
             @Override
-            public void fileChanged(String filename) throws Exception
+            public void fileChanged(String filename)
             {
                 queue.add(new Event(filename, Notification.CHANGED));
             }
 
             @Override
-            public void fileAdded(String filename) throws Exception
+            public void fileAdded(String filename)
             {
                 queue.add(new Event(filename, Notification.ADDED));
             }
@@ -201,7 +191,7 @@ public class ScannerTest
     public void testPatterns() throws Exception
     {
         //test include and exclude patterns
-        File root = new File(_directory, "proot");
+        File root = new File(_directory.toFile(), "proot");
         FS.ensureDirExists(root);
 
         File ttt = new File(root, "ttt.txt");
@@ -222,7 +212,7 @@ public class ScannerTest
         File y2 = new File(dir2, "yyy.txt");
         FS.touch(y2);
 
-        BlockingQueue<Event> queue = new LinkedBlockingQueue<Event>();
+        BlockingQueue<Event> queue = new LinkedBlockingQueue<>();
         //only scan the *.txt files for changes
         Scanner scanner = new Scanner();
         IncludeExcludeSet<PathMatcher, Path> pattern = scanner.addDirectory(root.toPath());
@@ -235,19 +225,19 @@ public class ScannerTest
         scanner.addListener(new Scanner.DiscreteListener()
         {
             @Override
-            public void fileRemoved(String filename) throws Exception
+            public void fileRemoved(String filename)
             {
                 queue.add(new Event(filename, Notification.REMOVED));
             }
 
             @Override
-            public void fileChanged(String filename) throws Exception
+            public void fileChanged(String filename)
             {
                 queue.add(new Event(filename, Notification.CHANGED));
             }
 
             @Override
-            public void fileAdded(String filename) throws Exception
+            public void fileAdded(String filename)
             {
                 queue.add(new Event(filename, Notification.ADDED));
             }
@@ -272,8 +262,7 @@ public class ScannerTest
     }
 
     @Test
-    @DisabledOnOs(WINDOWS) // TODO: needs review
-    @DisabledIfSystemProperty(named = "env", matches = "ci") // TODO: SLOW, needs review
+    @Tag("Slow")
     public void testAddedChangeRemove() throws Exception
     {
         touch("a0");
@@ -284,7 +273,7 @@ public class ScannerTest
 
         Event event = _queue.poll();
         assertNotNull(event, "Event should not be null");
-        assertEquals(_directory + "/a0", event._filename);
+        assertEquals(_directory.resolve("a0").toString(), event._filename);
         assertEquals(Notification.ADDED, event._notification);
 
         // add 3 more files
@@ -296,9 +285,9 @@ public class ScannerTest
         // not stable after 1 scan so should not be seen yet.
         _scanner.scan();
         event = _queue.poll();
-        assertTrue(event == null);
+        assertNull(event);
 
-        // Keep a2 unstable and remove a3 before it stabalized
+        // Keep a2 unstable and remove a3 before it stabilized
         Thread.sleep(1100); // make sure time in seconds changes
         touch("a2");
         delete("a3");
@@ -306,16 +295,16 @@ public class ScannerTest
         // only a1 is stable so it should be seen.
         _scanner.scan();
         event = _queue.poll();
-        assertTrue(event != null);
-        assertEquals(_directory + "/a1", event._filename);
+        assertNotNull(event);
+        assertEquals(_directory.resolve("a1").toString(), event._filename);
         assertEquals(Notification.ADDED, event._notification);
         assertTrue(_queue.isEmpty());
 
         // Now a2 is stable
         _scanner.scan();
         event = _queue.poll();
-        assertTrue(event != null);
-        assertEquals(_directory + "/a2", event._filename);
+        assertNotNull(event);
+        assertEquals(_directory.resolve("a2").toString(), event._filename);
         assertEquals(Notification.ADDED, event._notification);
         assertTrue(_queue.isEmpty());
 
@@ -328,7 +317,7 @@ public class ScannerTest
         // not stable after 1scan so nothing should not be seen yet.
         _scanner.scan();
         event = _queue.poll();
-        assertTrue(event == null);
+        assertNull(event);
 
         // Keep a2 unstable
         Thread.sleep(1100); // make sure time in seconds changes
@@ -337,16 +326,16 @@ public class ScannerTest
         // only a1 is stable so it should be seen.
         _scanner.scan();
         event = _queue.poll();
-        assertTrue(event != null);
-        assertEquals(_directory + "/a1", event._filename);
+        assertNotNull(event);
+        assertEquals(_directory.resolve("a1").toString(), event._filename);
         assertEquals(Notification.CHANGED, event._notification);
         assertTrue(_queue.isEmpty());
 
         // Now a2 is stable
         _scanner.scan();
         event = _queue.poll();
-        assertTrue(event != null);
-        assertEquals(_directory + "/a2", event._filename);
+        assertNotNull(event);
+        assertEquals(_directory.resolve("a2").toString(), event._filename);
         assertEquals(Notification.CHANGED, event._notification);
         assertTrue(_queue.isEmpty());
 
@@ -356,7 +345,7 @@ public class ScannerTest
         // not stable after 1scan so nothing should not be seen yet.
         _scanner.scan();
         event = _queue.poll();
-        assertTrue(event == null);
+        assertNull(event);
 
         // readd a2
         touch("a2");
@@ -364,22 +353,21 @@ public class ScannerTest
         // only a1 is stable so it should be seen.
         _scanner.scan();
         event = _queue.poll();
-        assertTrue(event != null);
-        assertEquals(_directory + "/a1", event._filename);
+        assertNotNull(event);
+        assertEquals(_directory.resolve("a1").toString(), event._filename);
         assertEquals(Notification.REMOVED, event._notification);
         assertTrue(_queue.isEmpty());
 
         // Now a2 is stable and is a changed file rather than a remove
         _scanner.scan();
         event = _queue.poll();
-        assertTrue(event != null);
-        assertEquals(_directory + "/a2", event._filename);
+        assertNotNull(event);
+        assertEquals(_directory.resolve("a2").toString(), event._filename);
         assertEquals(Notification.CHANGED, event._notification);
         assertTrue(_queue.isEmpty());
     }
 
     @Test
-    @DisabledOnOs(WINDOWS) // TODO: needs review
     public void testSizeChange() throws Exception
     {
         touch("tsc0");
@@ -388,13 +376,13 @@ public class ScannerTest
 
         // takes 2s to notice tsc0 and check that it is stable.  This syncs us with the scan
         Event event = _queue.poll();
-        assertTrue(event != null);
-        assertEquals(_directory + "/tsc0", event._filename);
+        assertNotNull(event);
+        assertEquals(_directory.resolve("tsc0").toString(), event._filename);
         assertEquals(Notification.ADDED, event._notification);
 
         // Create a new file by writing to it.
         long now = TimeUnit.NANOSECONDS.toMillis(System.nanoTime());
-        File file = new File(_directory, "st");
+        File file = new File(_directory.toFile(), "st");
         try (OutputStream out = new FileOutputStream(file, true))
         {
             out.write('x');
@@ -404,7 +392,7 @@ public class ScannerTest
             // Not stable yet so no notification.
             _scanner.scan();
             event = _queue.poll();
-            assertTrue(event == null);
+            assertNull(event);
 
             // Modify size only
             out.write('x');
@@ -414,13 +402,13 @@ public class ScannerTest
             // Still not stable yet so no notification.
             _scanner.scan();
             event = _queue.poll();
-            assertTrue(event == null);
+            assertNull(event);
 
             // now stable so finally see the ADDED
             _scanner.scan();
             event = _queue.poll();
-            assertTrue(event != null);
-            assertEquals(_directory + "/st", event._filename);
+            assertNotNull(event);
+            assertEquals(_directory.resolve("st").toString(), event._filename);
             assertEquals(Notification.ADDED, event._notification);
 
             // Modify size only
@@ -431,27 +419,26 @@ public class ScannerTest
             // Still not stable yet so no notification.
             _scanner.scan();
             event = _queue.poll();
-            assertTrue(event == null);
+            assertNull(event);
 
             // now stable so finally see the ADDED
             _scanner.scan();
             event = _queue.poll();
-            assertTrue(event != null);
-            assertEquals(_directory + "/st", event._filename);
+            assertNotNull(event);
+            assertEquals(_directory.resolve("st").toString(), event._filename);
             assertEquals(Notification.CHANGED, event._notification);
         }
     }
 
     private void delete(String string) throws IOException
     {
-        File file = new File(_directory, string);
-        if (file.exists())
-            IO.delete(file);
+        Path file = _directory.resolve(string);
+        Files.deleteIfExists(file);
     }
 
     private void touch(String string) throws IOException
     {
-        File file = new File(_directory, string);
+        File file = new File(_directory.toFile(), string);
         if (file.exists())
             file.setLastModified(TimeUnit.NANOSECONDS.toMillis(System.nanoTime()));
         else
