@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2021 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -129,9 +129,12 @@ public class HugeResourceTest
     @AfterAll
     public static void cleanupTestFiles()
     {
-        FS.ensureDeleted(staticBase);
-        FS.ensureDeleted(outputDir);
-        FS.ensureDeleted(multipartTempDir);
+        if (staticBase != null)
+            FS.ensureDeleted(staticBase);
+        if (outputDir != null)
+            FS.ensureDeleted(outputDir);
+        if (multipartTempDir != null)
+            FS.ensureDeleted(multipartTempDir);
     }
 
     private static void makeStaticFile(Path staticFile, long size) throws IOException
