@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2020 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -18,8 +18,8 @@
 
 package org.eclipse.jetty.websocket.server;
 
-import java.io.IOException;
 import java.util.Iterator;
+import javax.servlet.ServletContext;
 
 import org.eclipse.jetty.http.pathmap.MappedResource;
 import org.eclipse.jetty.http.pathmap.PathMappings;
@@ -28,7 +28,6 @@ import org.eclipse.jetty.http.pathmap.RegexPathSpec;
 import org.eclipse.jetty.http.pathmap.ServletPathSpec;
 import org.eclipse.jetty.http.pathmap.UriTemplatePathSpec;
 import org.eclipse.jetty.util.component.ContainerLifeCycle;
-import org.eclipse.jetty.util.component.Dumpable;
 import org.eclipse.jetty.websocket.api.WebSocketException;
 import org.eclipse.jetty.websocket.api.WebSocketPolicy;
 import org.eclipse.jetty.websocket.servlet.ServletUpgradeRequest;
@@ -41,42 +40,30 @@ import org.eclipse.jetty.websocket.servlet.WebSocketCreator;
  * Only applicable if using {@link WebSocketUpgradeFilter}
  * </p>
  */
-public class NativeWebSocketConfiguration extends ContainerLifeCycle implements MappedWebSocketCreator, Dumpable
+public class NativeWebSocketConfiguration extends ContainerLifeCycle implements MappedWebSocketCreator
 {
     private final WebSocketServerFactory factory;
     private final PathMappings<WebSocketCreator> mappings = new PathMappings<>();
-    
-    public NativeWebSocketConfiguration()
+
+    public NativeWebSocketConfiguration(ServletContext context)
     {
-        this(new WebSocketServerFactory());
+        this(new WebSocketServerFactory(context));
     }
-    
+
     public NativeWebSocketConfiguration(WebSocketServerFactory webSocketServerFactory)
     {
         this.factory = webSocketServerFactory;
         addBean(this.factory);
+        addBean(this.mappings);
     }
-    
+
     @Override
     public void doStop() throws Exception
     {
-        mappings.reset();
+        mappings.removeIf((mapped) -> !(mapped.getResource() instanceof PersistedWebSocketCreator));
         super.doStop();
     }
-    
-    @Override
-    public String dump()
-    {
-        return ContainerLifeCycle.dump(this);
-    }
-    
-    @Override
-    public void dump(Appendable out, String indent) throws IOException
-    {
-        // TODO: show factory/mappings ?
-        mappings.dump(out, indent);
-    }
-    
+
     /**
      * Get WebSocketServerFactory being used.
      *
@@ -86,7 +73,7 @@ public class NativeWebSocketConfiguration extends ContainerLifeCycle implements
     {
         return this.factory;
     }
-    
+
     /**
      * Get the matching {@link MappedResource} for the provided target.
      *
@@ -97,7 +84,7 @@ public class NativeWebSocketConfiguration extends ContainerLifeCycle implements
     {
         return this.mappings.getMatch(target);
     }
-    
+
     /**
      * Used to configure the Default {@link WebSocketPolicy} used by all endpoints that
      * don't redeclare the values.
@@ -108,18 +95,29 @@ public class NativeWebSocketConfiguration extends ContainerLifeCycle implements
     {
         return this.factory.getPolicy();
     }
-    
+
     /**
      * Manually add a WebSocket mapping.
+     * <p>
+     * If mapping is added before this configuration is started, then it is persisted through
+     * stop/start of this configuration's lifecycle.  Otherwise it will be removed when
+     * this configuration is stopped.
+     * </p>
      *
      * @param pathSpec the pathspec to respond on
      * @param creator the websocket creator to activate on the provided mapping.
      */
+    @Override
     public void addMapping(PathSpec pathSpec, WebSocketCreator creator)
     {
-        mappings.put(pathSpec, creator);
+        WebSocketCreator wsCreator = creator;
+        if (!isRunning())
+        {
+            wsCreator = new PersistedWebSocketCreator(creator);
+        }
+        mappings.put(pathSpec, wsCreator);
     }
-    
+
     /**
      * Manually add a WebSocket mapping.
      *
@@ -132,18 +130,18 @@ public class NativeWebSocketConfiguration extends ContainerLifeCycle implements
     {
         if (spec instanceof org.eclipse.jetty.websocket.server.pathmap.ServletPathSpec)
         {
-            addMapping(new ServletPathSpec(spec.getPathSpec()), creator);
+            addMapping(new ServletPathSpec(spec.getSpec()), creator);
         }
         else if (spec instanceof org.eclipse.jetty.websocket.server.pathmap.RegexPathSpec)
         {
-            addMapping(new RegexPathSpec(spec.getPathSpec()), creator);
+            addMapping(new RegexPathSpec(spec.getSpec()), creator);
         }
         else
         {
             throw new RuntimeException("Unsupported (Deprecated) PathSpec implementation type: " + spec.getClass().getName());
         }
     }
-    
+
     /**
      * Manually add a WebSocket mapping.
      *
@@ -154,19 +152,15 @@ public class NativeWebSocketConfiguration extends ContainerLifeCycle implements
      */
     public void addMapping(PathSpec pathSpec, final Class<?> endpointClass)
     {
-        mappings.put(pathSpec, new WebSocketCreator()
+        mappings.put(pathSpec, (req, resp) ->
         {
-            @Override
-            public Object createWebSocket(ServletUpgradeRequest req, ServletUpgradeResponse resp)
+            try
             {
-                try
-                {
-                    return endpointClass.newInstance();
-                }
-                catch (InstantiationException | IllegalAccessException e)
-                {
-                    throw new WebSocketException("Unable to create instance of " + endpointClass.getName(), e);
-                }
+                return endpointClass.getDeclaredConstructor().newInstance();
+            }
+            catch (Exception e)
+            {
+                throw new WebSocketException("Unable to create instance of " + endpointClass.getName(), e);
             }
         });
     }
@@ -243,10 +237,25 @@ public class NativeWebSocketConfiguration extends ContainerLifeCycle implements
         addMapping(pathSpec, endpointClass);
     }
 
-    @Override
-    public org.eclipse.jetty.websocket.server.pathmap.PathMappings<WebSocketCreator> getMappings()
+    private class PersistedWebSocketCreator implements WebSocketCreator
     {
-        throw new IllegalStateException("Access to PathMappings cannot be supported. See alternative API in javadoc for "
-                + MappedWebSocketCreator.class.getName());
+        private final WebSocketCreator delegate;
+
+        public PersistedWebSocketCreator(WebSocketCreator delegate)
+        {
+            this.delegate = delegate;
+        }
+
+        @Override
+        public Object createWebSocket(ServletUpgradeRequest req, ServletUpgradeResponse resp)
+        {
+            return delegate.createWebSocket(req, resp);
+        }
+
+        @Override
+        public String toString()
+        {
+            return "Persisted[" + super.toString() + "]";
+        }
     }
 }
