@@ -1,6 +1,6 @@
 //
 //  ========================================================================
-//  Copyright (c) 1995-2020 Mort Bay Consulting Pty Ltd and others.
+//  Copyright (c) 1995-2022 Mort Bay Consulting Pty Ltd and others.
 //  ------------------------------------------------------------------------
 //  All rights reserved. This program and the accompanying materials
 //  are made available under the terms of the Eclipse Public License v1.0
@@ -29,28 +29,29 @@ import org.eclipse.jetty.io.ByteBufferPool;
 import org.eclipse.jetty.io.MappedByteBufferPool;
 import org.eclipse.jetty.server.Request;
 import org.eclipse.jetty.server.handler.HandlerWrapper;
+import org.eclipse.jetty.servlet.ServletContextHandler;
 import org.eclipse.jetty.websocket.servlet.WebSocketCreator;
 
 public class WebSocketUpgradeHandlerWrapper extends HandlerWrapper implements MappedWebSocketCreator
 {
     private NativeWebSocketConfiguration configuration;
 
-    public WebSocketUpgradeHandlerWrapper()
+    public WebSocketUpgradeHandlerWrapper(ServletContextHandler context)
     {
-        this(new MappedByteBufferPool());
+        this(context, new MappedByteBufferPool());
     }
-    
-    public WebSocketUpgradeHandlerWrapper(ByteBufferPool bufferPool)
+
+    public WebSocketUpgradeHandlerWrapper(ServletContextHandler context, ByteBufferPool bufferPool)
     {
-        this.configuration = new NativeWebSocketConfiguration(new WebSocketServerFactory(bufferPool));
+        this.configuration = new NativeWebSocketConfiguration(new WebSocketServerFactory(context.getServletContext(), bufferPool));
     }
-    
+
     @Override
     public void addMapping(PathSpec spec, WebSocketCreator creator)
     {
         this.configuration.addMapping(spec, creator);
     }
-    
+
     /**
      * Add a mapping.
      *
@@ -77,39 +78,32 @@ public class WebSocketUpgradeHandlerWrapper extends HandlerWrapper implements Ma
         return configuration.removeMapping(spec);
     }
 
-    @Override
-    public org.eclipse.jetty.websocket.server.pathmap.PathMappings<WebSocketCreator> getMappings()
-    {
-        throw new IllegalStateException("Access to PathMappings cannot be supported. See alternative API in javadoc for "
-                + MappedWebSocketCreator.class.getName());
-    }
-    
     @Override
     public WebSocketCreator getMapping(String target)
     {
         return configuration.getMapping(target);
     }
-    
+
     @Override
     public void handle(String target, Request baseRequest, HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException
     {
-        if (configuration.getFactory().isUpgradeRequest(request,response))
+        if (configuration.getFactory().isUpgradeRequest(request, response))
         {
             MappedResource<WebSocketCreator> resource = configuration.getMatch(target);
             if (resource == null)
             {
                 // no match.
-                response.sendError(HttpServletResponse.SC_NOT_FOUND,"No websocket endpoint matching path: " + target);
+                response.sendError(HttpServletResponse.SC_NOT_FOUND, "No websocket endpoint matching path: " + target);
                 return;
             }
 
             WebSocketCreator creator = resource.getResource();
 
             // Store PathSpec resource mapping as request attribute
-            request.setAttribute(PathSpec.class.getName(),resource);
+            request.setAttribute(PathSpec.class.getName(), resource);
 
             // We have an upgrade request
-            if (configuration.getFactory().acceptWebSocket(creator,request,response))
+            if (configuration.getFactory().acceptWebSocket(creator, request, response))
             {
                 // We have a socket instance created
                 return;
@@ -124,6 +118,6 @@ public class WebSocketUpgradeHandlerWrapper extends HandlerWrapper implements Ma
                 return;
             }
         }
-        super.handle(target,baseRequest,request,response);
+        super.handle(target, baseRequest, request, response);
     }
 }
